# Copyright (C) 2017 Roman Samoilenko <ttahabatt@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import asyncio
import unittest

from heralding.capabilities.vnc import Vnc, RFB_VERSION, VNC_AUTH
from heralding.reporting.reporting_relay import ReportingRelay


class VncTests(unittest.TestCase):

  def setUp(self):
    self.loop = asyncio.new_event_loop()
    asyncio.set_event_loop(None)

    self.reporting_relay = ReportingRelay()
    self.reporting_relay_task = self.loop.run_in_executor(
        None, self.reporting_relay.start)

  def tearDown(self):
    self.reporting_relay.stop()
    # We give reporting_relay a chance to be finished
    self.loop.run_until_complete(self.reporting_relay_task)

    self.server.close()
    self.loop.run_until_complete(self.server.wait_closed())

    self.loop.close()

  def test_vnc_authentication(self):

    async def vnc_auth():
      reader, writer = await asyncio.open_connection(
          '127.0.0.1', 8888, loop=self.loop)
      # server rfb version
      _ = await reader.readline()
      writer.write(RFB_VERSION)

      # available auth methods
      _ = await reader.read(1024)
      writer.write(VNC_AUTH)

      # challenge
      _ = await reader.read(1024)
      # Pretending, that we encrypt received challenge with DES and send back the result.
      client_response = os.urandom(16)
      writer.write(client_response)

      # security result
      _ = await reader.read(1024)

    options = {'enabled': 'True', 'port': 8888, 'timeout': 30}
    capability = Vnc(options, self.loop)

    server_coro = asyncio.start_server(
        capability.handle_session, '0.0.0.0', 8888, loop=self.loop)
    self.server = self.loop.run_until_complete(server_coro)

    self.loop.run_until_complete(vnc_auth())



# We need this in order to reduce the number of third-party modules.
# It is a part of code from http.client, that adjusted to work with
# asyncio in our specific case.

import email.parser
from http.client import HTTPMessage


async def parse_headers(fp, _class=HTTPMessage):
  headers = []
  while True:
    line = await fp.readline()
    headers.append(line)
    if line in (b'\r\n', b'\n', b''):
      break
  hstring = b''.join(headers).decode('iso-8859-1')
  return email.parser.Parser(_class=_class).parsestr(hstring)



# Copyright (C) 2017 Johnny Vestergaard <jkv@unixcluster.dk>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import asyncio
import logging

from heralding.misc.session import Session

logger = logging.getLogger(__name__)


class HandlerBase:
  MAX_GLOBAL_SESSIONS = 800
  global_sessions = 0

  def __init__(self, options, loop):
    """
        Base class that all capabilities must inherit from.

        :param options: a dictionary of configuration options.

        """
    self.loop = loop
    self.options = options
    self.sessions = {}
    self.users = {}

    self.port = int(options['port'])
    if 'timeout' in options:
      self.timeout = options['timeout']
    else:
      self.timeout = 30

  def create_session(self, address, dest_address):
    protocol = self.__class__.__name__.lower()
    session = Session(address[0], address[1], protocol, self.users,
                      dest_address[1], dest_address[0])
    self.sessions[session.id] = session
    HandlerBase.global_sessions += 1
    logger.debug('Accepted %s session on %s:%s from %s:%s. (%s)', protocol,
                 dest_address[0], dest_address[1], address[0], address[1],
                 str(session.id))
    logger.debug('Size of session list for %s: %s', protocol,
                 len(self.sessions))
    return session

  def close_session(self, session):
    logger.debug('Closing session. (%s)', str(session.id))
    session.end_session()
    if session.id in self.sessions:
      del self.sessions[session.id]
    else:
      assert False
    HandlerBase.global_sessions -= 1

  async def execute_capability(self, reader, writer, session):
    reader = None  # NOQA
    writer = None  # NOQA
    session = None  # NOQA
    raise Exception("Must be implemented by child")

  async def handle_session(self, reader, writer):
    address = writer.get_extra_info('peername')
    dest_address = writer.get_extra_info('sockname')

    if HandlerBase.global_sessions > HandlerBase.MAX_GLOBAL_SESSIONS:
      protocol = self.__class__.__name__.lower()
      logger.warning(
          'Got %s session on port %s from %s:%s, but not handling it because the global session limit has '
          'been reached', protocol, self.port, *address)
    else:
      session = self.create_session(address, dest_address)
      try:
        await asyncio.wait_for(
            self.execute_capability(reader, writer, session),
            timeout=self.timeout,
            loop=self.loop)
      except asyncio.TimeoutError:
        logger.debug('Session timed out. (%s)', session.id)
      except (BrokenPipeError, ConnectionError) as err:
        logger.debug('Unexpected end of session: %s, errno: %s. (%s)', err,
                     err.errno, session.id)
      except (UnicodeDecodeError, KeyboardInterrupt):
        pass
      finally:
        self.close_session(session)
        if not self.loop.is_closed():
          writer.close()



import asyncio
import unittest

import psycopg2
from heralding.capabilities import postgresql
from heralding.misc.common import cancel_all_pending_tasks
from heralding.reporting.reporting_relay import ReportingRelay


class PostgreSQLTests(unittest.TestCase):

  def setUp(self):
    self.loop = asyncio.new_event_loop()
    asyncio.set_event_loop(None)

    self.reporting_relay = ReportingRelay()
    self.reporting_relay_task = self.loop.run_in_executor(
        None, self.reporting_relay.start)

  def tearDown(self):
    self.reporting_relay.stop()
    # We give reporting_relay a chance to be finished
    self.loop.run_until_complete(self.reporting_relay_task)

    self.server.close()
    self.loop.run_until_complete(self.server.wait_closed())

    self.loop.run_until_complete(cancel_all_pending_tasks(self.loop))
    self.loop.close()

  def test_invalid_login(self):
    """Tests if postgres server responds correctly to a invalid login attempt."""

    def postgresql_login():
      try:
        psycopg2.connect("postgres://scott:tiger@0.0.0.0:2504/")
      except psycopg2.OperationalError as e:
        return e
      return None

    options = {'enabled': 'True', 'port': 2504}
    postgresql_cap = postgresql.PostgreSQL(options, self.loop)

    server_coro = asyncio.start_server(
        postgresql_cap.handle_session, '0.0.0.0', 2504, loop=self.loop)
    self.server = self.loop.run_until_complete(server_coro)

    postgresql_task = self.loop.run_in_executor(None, postgresql_login)
    login_exception = self.loop.run_until_complete(postgresql_task)

    self.assertIsInstance(login_exception, psycopg2.OperationalError)
    self.assertEqual(
        str(login_exception),
        'FATAL:  password authentication failed for user "scott"\n')

  def cb(self, socket, command, option):
    return



# Copyright (C) 2017 Johnny Vestergaard <jkv@unixcluster.dk>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import syslog
import logging

from heralding.reporting.base_logger import BaseLogger

logger = logging.getLogger(__name__)


class SyslogLogger(BaseLogger):

  def __init__(self):
    super().__init__()
    logger.debug('Syslog logger started')

  def handle_auth_log(self, data):
    # for now this logger only handles authentication attempts where we are able
    # to log both username and password
    if 'username' in data and 'password' in data:
      message = "Authentication from {0}:{1}, with username: {2} " \
              "and password: {3}.".format(data['source_ip'], data['source_port'],
                                          data['username'], data['password'])
      syslog.syslog(syslog.LOG_ALERT, message)



# Copyright (C) 2017 Johnny Vestergaard <jkv@unixcluster.dk>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import asyncio
import unittest

import telnetlib

from heralding.capabilities import telnet
from heralding.misc.common import cancel_all_pending_tasks
from heralding.reporting.reporting_relay import ReportingRelay


class TelnetTests(unittest.TestCase):

  def setUp(self):
    self.loop = asyncio.new_event_loop()
    asyncio.set_event_loop(None)

    self.reporting_relay = ReportingRelay()
    self.reporting_relay_task = self.loop.run_in_executor(
        None, self.reporting_relay.start)

  def tearDown(self):
    self.reporting_relay.stop()
    # We give reporting_relay a chance to be finished
    self.loop.run_until_complete(self.reporting_relay_task)

    self.server.close()
    self.loop.run_until_complete(self.server.wait_closed())

    self.loop.run_until_complete(cancel_all_pending_tasks(self.loop))
    self.loop.close()

  def test_invalid_login(self):
    """Tests if telnet server responds correctly to a invalid login attempt."""

    def telnet_login():
      client = telnetlib.Telnet('localhost', 2503)
      # set this to 1 if having problems with this test
      client.set_debuglevel(0)
      # this disables all command negotiation.
      client.set_option_negotiation_callback(self.cb)
      # Expect username as first output

      reply = client.read_until(b'Username: ', 1)
      self.assertEqual(b'Username: ', reply)

      client.write(b'someuser' + b'\r\n')
      reply = client.read_until(b'Password: ', 5)
      self.assertTrue(reply.endswith(b'Password: '))

      client.write(b'somepass' + b'\r\n')
      reply = client.read_until(b'\n', 5)
      self.assertTrue(b'\n' in reply)

      client.close()

    options = {
        'enabled': 'True',
        'port': 2503,
        'protocol_specific_data': {
            'max_attempts': 3
        },
        'users': {
            'test': 'test'
        }
    }
    telnet_cap = telnet.Telnet(options, self.loop)

    server_coro = asyncio.start_server(
        telnet_cap.handle_session, '0.0.0.0', 2503, loop=self.loop)
    self.server = self.loop.run_until_complete(server_coro)

    telnet_task = self.loop.run_in_executor(None, telnet_login)
    self.loop.run_until_complete(telnet_task)

  def cb(self, socket, command, option):
    return



# Copyright (C) 2017 Johnny Vestergaard <jkv@unixcluster.dk>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import zmq
import queue
import logging

import heralding.misc
from heralding.misc.socket_names import SocketNames

logger = logging.getLogger(__name__)


class ReportingRelay:
  _logQueue = None

  def __init__(self):
    # we are singleton
    assert ReportingRelay._logQueue is None
    ReportingRelay._logQueue = queue.Queue(maxsize=10000)

    self.enabled = True

    context = heralding.misc.zmq_context
    self.internalReportingPublisher = context.socket(zmq.PUB)

  @staticmethod
  def logAuthAttempt(data):
    ReportingRelay._logQueue.put({'message_type': 'auth', 'content': data})

  @staticmethod
  def logSessionInfo(data):
    if ReportingRelay._logQueue is not None:
      ReportingRelay._logQueue.put({
          'message_type': 'session_info',
          'content': data
      })

  @staticmethod
  def logListenPorts(data):
    if ReportingRelay._logQueue is not None:
      ReportingRelay._logQueue.put({
          'message_type': 'listen_ports',
          'content': data
      })

  @staticmethod
  def logAuxiliaryData(data):
    if ReportingRelay._logQueue is not None:
      ReportingRelay._logQueue.put({
          'message_type': 'aux_info',
          'content': data
      })

  def start(self):
    self.internalReportingPublisher.bind(SocketNames.INTERNAL_REPORTING.value)

    while self.enabled or ReportingRelay._logQueue.qsize() > 0:
      try:
        data = ReportingRelay._logQueue.get(timeout=0.5)
        self.internalReportingPublisher.send_pyobj(data)
      except queue.Empty:
        pass

    # None signals 'going down' to listeners
    self.internalReportingPublisher.send_pyobj(None)
    self.internalReportingPublisher.close()

    # None is also used to signal we are all done
    ReportingRelay._logQueue = None

  def stop(self):
    self.enabled = False



# Copyright (C) 2017 Johnny Vestergaard <jkv@unixcluster.dk>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import json
import uuid
import logging
from datetime import datetime

import heralding.honeypot
from heralding.reporting.reporting_relay import ReportingRelay

logger = logging.getLogger(__name__)


class Session:

  def __init__(self,
               source_ip,
               source_port,
               protocol,
               users,
               destination_port=None,
               destination_ip=''):

    self.id = uuid.uuid4()
    self.source_ip = source_ip
    self.source_port = source_port
    self.protocol = protocol
    if heralding.honeypot.Honeypot.public_ip:
      self.destination_ip = heralding.honeypot.Honeypot.public_ip
    else:
      self.destination_ip = destination_ip
    self.destination_port = destination_port
    self.timestamp = datetime.utcnow()
    self.num_ = 0
    self.session_ended = False
    # protocol specific data
    self.auxiliary_data = {}

    self.connected = True

    # for session specific volatile data (will not get logged)
    self.vdata = {}

    self.auth_attempts = []

    self.last_activity = datetime.utcnow()
    self.log_start_session()

  def log_start_session(self):
    entry = self.get_session_info(False)
    ReportingRelay.logSessionInfo(entry)

  def activity(self):
    self.last_activity = datetime.utcnow()

  def is_connected(self):
    return self.connected

  def get_auxiliary_data(self):
    return {}

  def get_number_of_login_attempts(self):
    return len(self.auth_attempts)

  def add_auth_attempt(self, _type, **kwargs):

    # constructs dict to transmitted right away.
    entry = {
        'timestamp': datetime.utcnow().strftime('%Y-%m-%d %H:%M:%S.%f'),
        'session_id': str(self.id),
        'auth_id': str(uuid.uuid4()),
        'source_ip': self.source_ip,
        'source_port': self.source_port,
        'destination_ip': self.destination_ip,
        'destination_port': self.destination_port,
        'protocol': self.protocol,
        'username': None,
        'password': None,
        'password_hash': None
    }
    if 'username' in kwargs:
      entry['username'] = kwargs['username']
    if 'password' in kwargs:
      entry['password'] = kwargs['password']
    if 'password_hash' in kwargs:
      entry['password_hash'] = kwargs['password_hash']
    ReportingRelay.logAuthAttempt(entry)

    # add to internal dict used for reporting when the session ends
    self.auth_attempts.append({
        'timestamp': entry['timestamp'],
        'username': entry['username'],
        'password': entry['password'],
    })

    self.activity()
    logger.debug(
        '%s authentication attempt from %s:%s. Auth mechanism: %s, session id %s '
        'Credentials: %s', self.protocol, self.source_ip, self.source_port,
        _type, self.id, json.dumps(kwargs))

  def get_session_info(self, session_ended):
    entry = {
        'timestamp': self.timestamp.strftime('%Y-%m-%d %H:%M:%S.%f'),
        'duration': int((datetime.utcnow() - self.timestamp).total_seconds()),
        'session_id': str(self.id),
        'source_ip': self.source_ip,
        'source_port': self.source_port,
        'destination_ip': self.destination_ip,
        'destination_port': self.destination_port,
        'protocol': self.protocol,
        'num_auth_attempts': len(self.auth_attempts),
        'auth_attempts': self.auth_attempts,
        'session_ended': session_ended,
        'auxiliary_data': self.auxiliary_data
    }
    return entry

  def set_auxiliary_data(self, data):
    self.auxiliary_data = data

  def end_session(self):
    if not self.session_ended:
      self.session_ended = True
      self.connected = False
      entry = self.get_session_info(True)

      ReportingRelay.logSessionInfo(entry)
      logger.debug('Session with session id %s ended', self.id)



# Copyright (C) 2019 Sudipta Pandit <realsdx@protonmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import struct
import logging

from heralding.capabilities.handlerbase import HandlerBase

logger = logging.getLogger(__name__)


# Implemented according to https://dev.mysql.com/doc/internals/en/connection-phase-packets.html
class MySQL(HandlerBase):

  def __init__(self, options, loop):
    super().__init__(options, loop)
    self.PROTO_VER = b'\x0a'
    self.SERVER_VER = b'5.7.16\x00'
    self.AUTH_PLUGIN = b'mysql_native_password\x00'  # SHA-1 hash of password

  @staticmethod
  def convert4To3Byte(num):
    # MySQL protocol requires 3 byte integers for payload length
    return struct.pack("<I", num)[:3]

  def server_greeting(self):
    # Server Greeting(HandshakeV10) Packet:

    # 0x00 0x03    Payload length
    # 0x03 0x01    Sequence No
    # 0x04 0x01    Protocol version (0x0A)
    # 0x05 0x07    MySQL version (5.7.16\0 in this case, variable length in general)
    # 0x0C 0x04    Thread ID
    # 0x10 0x09    Salt (8 random bytes + \0)
    # 0x19 0x02    Server capabilities (0xFF 0x7F)
    # 0x1B 0x01    Server language (0x21 = utf8 COLLATE utf8_general_ci)
    # 0x1C 0x02    Server status (0x02 0x00)
    # 0x1E 0x02    Extended server capabilities (0xFF 0x81)
    # 0x20 0x01    Authentication plugin length
    # 0x21 0x0A    Unused (zeros)
    # 0x2B 0x0D    Salt (12 random bytes + \0)
    # 0x38 0x16    Authentication plugin ('mysql_native_password\0')

    # NB: actual payload without the first four bytes (Payload lenght + sequence no)

    # packet_length including payload_length+sequence_no
    packet_length = 0x3+0x1+0x1+len(self.SERVER_VER)+0x04+(0x08+0x01)+0x02+0x01 + \
        0x02+0x02+0x01+0x0A+0x0D+len(self.AUTH_PLUGIN)

    payload_len = MySQL.convert4To3Byte(packet_length - 0x04)
    seq_no = b'\x00'  # Always will be first packet
    thread_id = struct.pack("<I", 4321)
    salt_1 = os.urandom(8) + b'\x00'
    server_cap = b'\xFF\xF7'
    server_lang = b'\x21'
    server_status = b'\x02\x00'
    ext_server_cap = b'\xFF\x81'
    auth_plugin_len = bytes([len(self.AUTH_PLUGIN) - 1])
    zeros = bytes(0x0A)
    salt_2 = os.urandom(12) + b'\x00'

    packet = payload_len+seq_no+self.PROTO_VER+self.SERVER_VER+thread_id +\
        salt_1+server_cap+server_lang+server_status+ext_server_cap+auth_plugin_len +\
        zeros+salt_2+self.AUTH_PLUGIN

    return packet

  def auth_switch_request(self, seq_no):
    # Auth Switch Request Packet:

    # 0x00 0x03    Payload length
    # 0x03 0x01    Sequence No
    # 0x04 0x01    Auth switch request (0xFE)
    # 0x05 0x16    Auth method name (variable length, NUL-terminated)
    # 0x1B 0x15    Auth method data (20 bytes of random salt + \0)

    # Incase any client using other auth methods, request client to switch to 'mysql_native_password'

    packet_length = 3 + 1 + 1 + len(self.AUTH_PLUGIN) + 20 + 1
    payload_len = MySQL.convert4To3Byte(packet_length - 4)
    seq_no = bytes([seq_no])
    auth_switch_req = b'\xFE'
    auth_data = os.urandom(20) + b'\x00'

    packet = payload_len + seq_no + auth_switch_req + self.AUTH_PLUGIN + auth_data

    return packet

  def auth_failed(self, seq_no, user, server, using_password):
    # Auth Failed Packet(Error 1045):

    # 0x00 0x03    Payload length
    # 0x03 0x01    Sequence #
    # 0x04 0x01    Error packet (0xFF)
    # 0x05 0x02    Error code (1045, 0x0415)
    # 0x07 0x06    SQL State (#28000)
    # 0x0D  ??     Error message

    error_msg = bytes(
        "Access denied for user '{}'@'{}' (using password: {})".format(
            user, server, using_password), 'utf-8')
    full_length = 3 + 1 + 1 + 2 + 6 + len(error_msg)  # taking out null

    payload_len = MySQL.convert4To3Byte(full_length - 4)
    seq_no = bytes([seq_no])
    error_packet = b'\xFF'  # Error Packet ID
    error_code = b'\x15\x04'  # Error code 1045 (0x0415)
    sql_state = bytes("#28000", 'utf-8')

    packet = payload_len + seq_no + error_packet + error_code + sql_state + error_msg
    return packet

  async def execute_capability(self, reader, writer, session):
    try:
      await self._handle_session(reader, writer, session)
    except struct.error as exc:
      logger.debug('MySQL connection error: %s', exc)
      session.end_session()

  async def _handle_session(self, reader, writer, session):
    # HandshakeResponse41 Packet:

    # 0x00 0x03    Payload length
    # 0x03 0x01    Sequence No
    # 0x04 0x04    Client capability flags
    # 0x08 0x04    Max packet size
    # 0x0C 0x01    Client character set
    # 0x0D 0x17    Unused(zeros)
    # 0x24 ??      User name (NUL-terminated)
    # XX   0x01    Password length
    # XX+1 ??      Password
    # YY   ??      Schema (NUL-terminated)
    # ZZ   ??      Client authentication plugin (NUL-terminated)
    # RR   ??      Client connection attibutes

    address = writer.get_extra_info('peername')[0]
    writer.write(self.server_greeting())
    await writer.drain()
    data = await reader.read(2048)

    # Handle empty data and quit command
    if not data:
      logger.warning("Got no response from client. Ending the session")
      session.end_session()
      return
    elif data == b'\x01\x00\x00\x00\x01':
      logger.warning("Received COM_QUIT from client. Ending the session")
      session.end_session()
      return

    caps = int.from_bytes(data[0x04:0x08], byteorder='little')

    # Check if client version is above 4.1 , if not terminate session
    if not caps & 0x00000200:
      logger.warning(
          "Client vesrion should be 4.1 or above. Ending the session")
      session.end_session()
      return

    try:
      max_size = int.from_bytes(data[0x08:0x0C], byteorder='little')
      username_end_pos = data.index(b'\x00', 0x24, max_size - 0x24)
      username = str(data[0x24:username_end_pos], 'utf-8')
      password_len = data[username_end_pos + 1]
      using_password = "YES" if password_len > 0 else "NO"
      plugin_offset = username_end_pos + 1 + 1  # start offset of auth_plugin
      seq_no = 2  # if no auth_switch_request
      password_enc = ''

      if password_len > 0:
        # for logging coverted to printable hex
        password_enc = data[plugin_offset:plugin_offset + password_len].hex()
        plugin_offset = plugin_offset + password_len
    except ValueError:
      logger.warning("Malformed packet received. Ending the session")
      session.end_session()
      return

    # check if schema(db) is present
    if caps & 0x00000008:
      schema_offset = plugin_offset  # start offset of db
      try:
        schema_end_pos = data.index(b'\x00', schema_offset,
                                    max_size - schema_offset)
        plugin_offset = schema_end_pos + 1
      except ValueError:
        logger.warning("Could not find the schema. Ending the session")
        session.end_session()
        return

    # check if plugin_auth enabled
    if caps & 0x00080000:
      try:
        plugin_auth_offset = plugin_offset
        plugin_auth_pos = data.index(b'\x00', plugin_auth_offset,
                                     max_size - plugin_auth_offset)
        plugin_auth = data[plugin_auth_offset:plugin_auth_pos]
      except ValueError:
        logger.warning(
            "Cloud not find the plugin_auth data. Ending the session")
        session.end_session()
        return

      if "mysql_native_password" != str(plugin_auth, 'utf-8'):
        writer.write(self.auth_switch_request(seq_no))
        await writer.drain()
        if (await reader.read(1024)):
          seq_no = 4

    session.add_auth_attempt(
        'encrypted', username=username, password=password_enc)
    writer.write(self.auth_failed(seq_no, username, address, using_password))
    await writer.drain()



# Copyright (C) 2017 Johnny Vestergaard <jkv@unixcluster.dk>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import base64
import logging
import binascii

from heralding.capabilities.handlerbase import HandlerBase

logger = logging.getLogger(__name__)

CRLF = '\r\n'


class Imap(HandlerBase):

  def __init__(self, options, loop):
    super().__init__(options, loop)
    self.max_tries = int(self.options['protocol_specific_data']['max_attempts'])
    self.banner = self.options['protocol_specific_data']['banner']

    self.available_commands = [
        'authenticate', 'capability', 'login', 'logout', 'noop'
    ]
    self.available_mechanisms = ['plain']

  async def execute_capability(self, reader, writer, session):
    await self._handle_session(session, reader, writer)

  async def _handle_session(
      self,
      session,
      reader,
      writer,
  ):
    await self.send_message(writer, self.banner)

    state = "Not Authenticated"
    while state != 'Logout' and session.connected:
      # An exception is raised inside await reader.readline() in case of
      # sudden connection reset.
      raw_msg = await reader.readline()
      if not raw_msg:
        break

      raw_msg_str = str(raw_msg, 'utf-8')

      cmd_msg = raw_msg_str.rstrip().split(' ', 2)
      if len(cmd_msg) == 0:
        continue
      elif len(cmd_msg) == 1:
        await self.send_message(writer, "* BAD invalid command")
        continue
      elif len(cmd_msg) == 2:
        tag = cmd_msg[0]
        cmd = cmd_msg[1]
        args = ''
      else:
        tag = cmd_msg[0]
        cmd = cmd_msg[1]
        args = cmd_msg[2]

      cmd = cmd.lower()
      if cmd not in self.available_commands:
        await self.send_message(writer, tag + " BAD invalid command")
      else:
        func_to_call = getattr(self, 'cmd_{0}'.format(cmd), None)
        if func_to_call:
          return_value = await func_to_call(session, reader, writer, tag, args)
          state = return_value
        else:
          await self.send_message(writer, tag + " BAD invalid command")
    session.end_session()

  async def cmd_authenticate(self, session, reader, writer, tag, args):
    mechanism = args.split()
    if len(mechanism) == 1:
      auth_mechanism = mechanism[0].lower()
    else:
      await self.send_message(writer, tag + ' BAD invalid command')
      return "Not Authenticated"

    if auth_mechanism in self.available_mechanisms:
      # the space after '+' is needed according to RFC
      await self.send_message(writer, '+ ')
      raw_msg = await reader.read(512)

      if auth_mechanism == 'plain':
        success, credentials = self.try_b64decode(raw_msg, session)
        # \x00 is a separator between authorization identity,
        # username and password. Authorization identity isn't used in
        # this auth mechanism, so we must have 2 \x00 symbols.(RFC 4616)
        if success and credentials.count('\x00') == 2:
          raw_msg_dec = str(base64.b64decode(raw_msg), 'utf-8')
          _, user, password = raw_msg_dec.split('\x00')
          session.add_auth_attempt(
              'plaintext', username=user, password=password)
          await self.send_message(writer, tag + ' NO Authentication failed')
        else:
          await self.send_message(writer, tag + ' BAD invalid command')
    else:
      await self.send_message(writer, tag + ' BAD invalid command')
    self.stop_if_too_many_attempts(session)
    return 'Not Authenticated'

  async def cmd_capability(self, session, reader, writer, tag, args):
    await self.send_message(writer, '* CAPABILITY IMAP4rev1 AUTH=PLAIN')
    await self.send_message(writer, tag + ' OK CAPABILITY completed')
    return 'Not Authenticated'

  async def cmd_login(self, session, reader, writer, tag, args):
    if args:
      user_cred = args.split(' ', 1)
    else:
      await self.send_message(writer, tag + ' BAD invalid command')
      return 'Not Authenticated'

    # Delete first and last quote,
    # because login and password can be sent as quoted strings
    if len(user_cred) == 1:
      user = self.strip_quotes(user_cred[0])
      password = ''
    else:
      user = self.strip_quotes(user_cred[0])
      password = self.strip_quotes(user_cred[1])

    session.add_auth_attempt('plaintext', username=user, password=password)
    await self.send_message(writer, tag + ' NO Authentication failed')
    self.stop_if_too_many_attempts(session)
    return 'Not Authenticated'

  async def cmd_logout(self, session, reader, writer, tag, args):
    await self.send_message(writer, '* BYE IMAP4rev1 Server logging out')
    await self.send_message(writer, tag + ' OK LOGOUT completed')
    return 'Logout'

  async def cmd_noop(self, session, reader, writer, tag, args):
    await self.send_message(writer, tag + ' OK NOOP completed')
    return 'Not Authenticated'

  def stop_if_too_many_attempts(self, session):
    if self.max_tries < session.get_number_of_login_attempts():
      session.end_session()

  @staticmethod
  async def send_message(writer, msg):
    message_bytes = bytes(msg + CRLF, 'utf-8')
    writer.write(message_bytes)
    await writer.drain()

  @staticmethod
  def try_b64decode(b64_str, session):
    try:
      result = base64.b64decode(b64_str)
      return True, str(result, 'utf-8')
    except binascii.Error:
      logger.warning('Error decoding base64: {0} ({1})'.format(
          binascii.hexlify(b64_str), session.id))
      return False, ''

  @staticmethod
  def strip_quotes(quoted_str):
    if quoted_str.startswith('\"') and quoted_str.endswith('\"'):
      nonquoted_str = quoted_str[1:-1]
    else:
      nonquoted_str = quoted_str
    return nonquoted_str



# Copyright (C) 2017 Johnny Vestergaard <jkv@unixcluster.dk>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import asyncio
import unittest

from heralding.capabilities.pop3 import Pop3
from heralding.misc.common import cancel_all_pending_tasks
from heralding.reporting.reporting_relay import ReportingRelay


class Pop3Tests(unittest.TestCase):

  def setUp(self):
    self.loop = asyncio.new_event_loop()
    asyncio.set_event_loop(None)

    self.reporting_relay = ReportingRelay()
    self.reporting_relay_task = self.loop.run_in_executor(
        None, self.reporting_relay.start)

  def tearDown(self):
    self.reporting_relay.stop()
    # We give reporting_relay a chance to be finished
    self.loop.run_until_complete(self.reporting_relay_task)

    self.server.close()
    self.loop.run_until_complete(self.server.wait_closed())

    self.loop.run_until_complete(cancel_all_pending_tasks(self.loop))

    self.loop.close()

  def test_login(self):
    """Testing different login combinations"""

    async def pop3_login():
      login_sequences = [
          # invalid login, invalid password
          (('USER wakkwakk', b'+OK User accepted'),
           ('PASS wakkwakk', b'-ERR Authentication failed.')),
          # PASS without user
          (
              ('PASS bond', b'-ERR No username given.'),),
          # Try to run a TRANSACITON state command in AUTHORIZATION state
          (
              ('RETR', b'-ERR Unknown command'),),
      ]
      for sequence in login_sequences:
        reader, writer = await asyncio.open_connection(
            '127.0.0.1', 8888, loop=self.loop)
        # skip banner
        await reader.readline()

        for pair in sequence:
          writer.write(bytes(pair[0] + "\r\n", 'utf-8'))
          response = await reader.readline()
          self.assertEqual(response.rstrip(), pair[1])

    options = {
        'port': 110,
        'protocol_specific_data': {
          'banner': '+OK POP3 server ready',
          'max_attempts': 3
        },
        'users': {
            'james': 'bond'
        }
    }
    sut = Pop3(options, self.loop)

    server_coro = asyncio.start_server(
        sut.handle_session, '0.0.0.0', 8888, loop=self.loop)
    self.server = self.loop.run_until_complete(server_coro)

    self.loop.run_until_complete(pop3_login())



#
# Some contents of this file is part of the PyRDP project.
# Copyright (C) 2018 GoSecure Inc.
# Licensed under the GPLv3 or later.
#

import struct


class Integer:
  FORMAT = ""

  @classmethod
  def pack(cls, value):
    data_bytes = struct.pack(cls.FORMAT, value)

    return data_bytes


# 8 bits
class Int8(Integer):
  FORMAT = "<b"


class Uint8(Integer):
  FORMAT = "<B"


# 16 bits
class Int16LE(Integer):
  FORMAT = "<h"


class Int16BE(Integer):
  FORMAT = ">h"


class Uint16LE(Integer):
  FORMAT = "<H"


class Uint16BE(Integer):
  FORMAT = ">H"


# 32 bits
class Int32LE(Integer):
  FORMAT = "<i"


class Int32BE(Integer):
  FORMAT = ">i"


class Uint32LE(Integer):
  FORMAT = "<I"


class Uint32BE(Integer):
  FORMAT = ">I"



# pylint: disable-msg=E1101
# Copyright (C) 2017 Johnny Vestergaard <jkv@unixcluster.dk>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import curses
import logging

from heralding.capabilities.handlerbase import HandlerBase
from heralding.libs.telnetsrv.telnetsrvlib import TelnetHandlerBase

logger = logging.getLogger(__name__)


class Telnet(HandlerBase):

  def __init__(self, options, loop):
    super().__init__(options, loop)
    TelnetWrapper.max_tries = int(
        self.options['protocol_specific_data']['max_attempts'])

  async def execute_capability(self, reader, writer, session):
    telnet_cap = TelnetWrapper(reader, writer, session, self.loop)
    await telnet_cap.run()


class TelnetWrapper(TelnetHandlerBase):
  """
    Wraps the telnetsrv module to fit the Honeypot architecture.
    """
  PROMPT = b'$ '
  max_tries = 3
  TERM = 'ansi'

  authNeedUser = True
  authNeedPass = True

  def __init__(self, reader, writer, session, loop):
    self.auth_count = 0
    self.username = None
    self.session = session
    address = writer.get_extra_info('address')
    super().__init__(reader, writer, address, loop)

  async def authentication_ok(self):
    while self.auth_count < TelnetWrapper.max_tries:
      username = await self.readline(prompt=b"Username: ", use_history=False)
      password = await self.readline(
          echo=False, prompt=b"Password: ", use_history=False)
      self.session.add_auth_attempt(
          _type='plaintext',
          username=str(username, 'utf-8'),
          password=str(password, 'utf-8'))
      if self.DOECHO:
        self.write(b"\n")
      self.auth_count += 1
    self.writeline(b'Username: ')  # It fixes a problem with Hydra bruteforcer.
    return False

  def setterm(self, term):
    # Dummy file for the purpose of tests.
    with open('/dev/null', 'w') as f:
      curses.setupterm(
          term, f.fileno())  # This will raise if the termtype is not supported
      self.TERM = term
      self.ESCSEQ = {}
      for k in self.KEYS.keys():
        str_ = curses.tigetstr(curses.has_key._capability_names[k])
        if str_:
          self.ESCSEQ[str_] = k
      self.CODES['DEOL'] = curses.tigetstr('el')
      self.CODES['DEL'] = curses.tigetstr('dch1')
      self.CODES['INS'] = curses.tigetstr('ich1')
      self.CODES['CSRLEFT'] = curses.tigetstr('cub1')
      self.CODES['CSRRIGHT'] = curses.tigetstr('cuf1')

  def session_end(self):
    self.session.end_session()



# Copyright (C) 2018 Roman Samoilenko <ttahabatt@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import logging

from heralding.capabilities.handlerbase import HandlerBase

# Socks constants
SOCKS_VERSION = b'\x05'
AUTH_METHOD = b'\x02'  # username/password authentication. RFC 1929
SOCKS_FAIL = b'\xFF'

logger = logging.getLogger(__name__)


class Socks5(HandlerBase):

  async def execute_capability(self, reader, writer, session):
    await self._handle_session(reader, writer, session)

  async def _handle_session(self, reader, writer, session):
    # 257 - max bytes number for greeting according to RFC 1928
    greeting = await reader.read(257)
    if len(greeting) > 2:
      await self.try_authenticate(reader, writer, session, greeting)
    else:
      logger.debug("Incorrect client greeting string: %r" % greeting)
    session.end_session()

  async def try_authenticate(self, reader, writer, session, greeting):
    version, authmethods = self.unpack_msg(greeting)
    if version == SOCKS_VERSION:
      if AUTH_METHOD in authmethods:
        await self.do_authenticate(reader, writer, session)
      else:
        writer.write(SOCKS_VERSION + SOCKS_FAIL)
        await writer.drain()
      session.set_auxiliary_data(self.get_auxiliary_data(authmethods))
    else:
      logger.debug("Wrong socks version: %r" % version)

  async def do_authenticate(self, reader, writer, session):
    writer.write(SOCKS_VERSION + AUTH_METHOD)
    await writer.drain()
    # 513 - max bytes number for username/password auth according to RFC 1929
    auth_data = await reader.read(513)
    if len(auth_data) > 2:
      username, password = self.unpack_auth(auth_data)
      session.add_auth_attempt(
          'plaintext', username=username.decode(), password=password.decode())
      writer.write(AUTH_METHOD + SOCKS_FAIL)
      await writer.drain()
    else:
      logger.debug("Wrong authentication data: %r" % auth_data)

  def get_auxiliary_data(self, authmethods):
    _methods = []
    for m in authmethods:
      if m == 2:
        _methods.append("USERNAME/PASSWORD")
      elif m == 0:
        _methods.append("NO AUTHENTICATION REQUIRED")
      elif m == 1:
        _methods.append("GSSAPI")
      elif 3 <= m <= 127:
        _methods.append("IANA ASSIGNED(%s)" % hex(m))
      elif 128 <= m <= 254:
        _methods.append("PRIVATE METHOD(%s)" % hex(m))
      elif m == 255:
        _methods.append("NO ACCEPTABLE METHODS")

    data = {"client_auth_methods": _methods}
    return data

  @staticmethod
  def unpack_msg(data):
    socks_version = data[:1]  # we need byte representation
    authmethods_n = data[1]
    authmethods = data[2:2 + authmethods_n]
    return socks_version, authmethods

  @staticmethod
  def unpack_auth(auth_data):
    ulen = auth_data[1]
    username = auth_data[2:2 + ulen]
    password_part = auth_data[2 + ulen:]
    if password_part:
      plen = password_part[0]
      password = auth_data[-plen:]
    else:
      password = b""
    return username, password



# Copyright (C) 2017 Johnny Vestergaard <jkv@unixcluster.dk>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import logging
import hpfeeds
import json

from heralding.reporting.base_logger import BaseLogger

logger = logging.getLogger(__name__)


class HpFeedsLogger(BaseLogger):

  def __init__(self, session_channel, auth_channel, host, port, ident, secret):
    super().__init__()
    self.session_channel = session_channel
    self.auth_channel = auth_channel
    self.host = host
    self.port = port
    self.ident = ident
    self.secret = secret
    self._initial_connection_happend = False
    logger.info('HpFeeds logger started.')

  def start(self):
    if not self._initial_connection_happend:
      self.hp_connection = hpfeeds.new(self.host, self.port, self.ident,
                                       self.secret, True)
      self._initial_connection_happend = True
      logger.info('HpFeeds logger connected to %s:%s.', self.host, self.port)
    # after we established that we can connect enter the subscribe and enter the polling loop
    super().start()

  def loggerStopped(self):
    self.stop()
    self.close()

  def handle_auth_log(self, data):
    if self._initial_connection_happend:
      self.hp_connection.publish(self.auth_channel, json.dumps(data).encode())

  def handle_session_log(self, data):
    if self._initial_connection_happend:
      self.hp_connection.publish(self.session_channel,
                                 json.dumps(data).encode())



# Copyright (C) 2017 Johnny Vestergaard <jkv@unixcluster.dk>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import logging

from heralding.capabilities.handlerbase import HandlerBase

logger = logging.getLogger(__name__)


class Pop3(HandlerBase):
  max_tries = 10
  cmds = {}

  def __init__(self, options, loop):
    super().__init__(options, loop)
    Pop3.max_tries = int(self.options['protocol_specific_data']['max_attempts'])
    self.banner = self.options['protocol_specific_data']['banner']

  async def execute_capability(self, reader, writer, session):
    await self._handle_session(session, reader, writer)

  async def _handle_session(self, session, reader, writer):
    await self.send_message(writer, self.banner)

    state = 'AUTHORIZATION'
    while state != '' and session.connected:
      raw_msg = await reader.readline()
      if not raw_msg:
        break

      raw_msg_str = str(raw_msg, 'utf-8')

      session.activity()
      cmd_msg = raw_msg_str.rstrip().split(' ', 1)
      if len(cmd_msg) == 0:
        continue
      elif len(cmd_msg) == 1:
        cmd = cmd_msg[0]
        msg = ''
      else:
        cmd = cmd_msg[0]
        msg = cmd_msg[1]

      cmd = cmd.lower()

      if cmd not in ['user', 'pass', 'quit', 'noop']:
        await self.send_message(writer, '-ERR Unknown command')
      else:
        func_to_call = getattr(self, 'cmd_{0}'.format(cmd), None)
        return_value = await func_to_call(session, reader, writer, msg)
        # state changers!
        if state == 'AUTHORIZATION' or cmd == 'quit':
          state = return_value

    session.end_session()

  # APOP mrose c4c9334bac560ecc979e58001b3e22fb
  # +OK mrose's maildrop has 2 messages (320 octets)
  def auth_apop(self, session, gsocket, msg):
    raise Exception('Not implemented yet!')

  # USER mrose
  # +OK User accepted
  # PASS tanstaaf
  # +OK Pass accepted
  # or: "-ERR Authentication failed."
  # or: "-ERR No username given."
  async def cmd_user(self, session, reader, writer, msg):
    session.vdata['USER'] = msg
    await self.send_message(writer, '+OK User accepted')
    return 'AUTHORIZATION'

  async def cmd_pass(self, session, reader, writer, msg):
    if 'USER' not in session.vdata:
      await self.send_message(writer, '-ERR No username given.')
    else:
      session.add_auth_attempt(
          'plaintext', username=session.vdata['USER'], password=msg)
      await self.send_message(writer, "-ERR Authentication failed.")

    if 'USER' in session.vdata:
      del session.vdata['USER']
    return 'AUTHORIZATION'

  async def cmd_noop(self, session, reader, writer, msg):
    await self.send_message(writer, '+OK')

  async def cmd_quit(self, session, reader, writer, msg):
    await self.send_message(writer, '+OK Logging out')
    return ''

  @staticmethod
  async def send_message(writer, msg):
    message_bytes = bytes(msg + "\n", 'utf-8')
    writer.write(message_bytes)
    await writer.drain()



# Copyright (C) 2018 Roman Samoilenko <ttahabatt@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import asyncio
import unittest

import heralding.capabilities.socks5 as socks
from heralding.reporting.reporting_relay import ReportingRelay


class Socks5Tests(unittest.TestCase):

  def setUp(self):
    self.loop = asyncio.new_event_loop()
    asyncio.set_event_loop(None)

    self.reporting_relay = ReportingRelay()
    self.reporting_relay_task = self.loop.run_in_executor(
        None, self.reporting_relay.start)

  def tearDown(self):
    self.reporting_relay.stop()
    # We give reporting_relay a chance to be finished
    self.loop.run_until_complete(self.reporting_relay_task)

    self.server.close()
    self.loop.run_until_complete(self.server.wait_closed())

    self.loop.close()

  def test_socks_authentication(self):

    async def socks_auth():
      reader, writer = await asyncio.open_connection(
          '127.0.0.1', 8888, loop=self.loop)

      # Greeting to the server. version+authmethod number+authmethod
      client_greeting = socks.SOCKS_VERSION + b"\x01" + socks.AUTH_METHOD
      writer.write(client_greeting)

      # Receive version+chosen authmethod
      _ = await reader.read(2)

      # Send credentials.
      # version+username len+username+password len+password
      credentials = b"\x05\x08username\x08password"
      writer.write(credentials)

      # Receive authmethod+\xff
      res = await reader.read(2)
      self.assertEqual(res, socks.AUTH_METHOD + socks.SOCKS_FAIL)

    options = {'enabled': 'True', 'port': 8888, 'timeout': 30}
    capability = socks.Socks5(options, self.loop)

    server_coro = asyncio.start_server(
        capability.handle_session, '127.0.0.1', 8888, loop=self.loop)
    self.server = self.loop.run_until_complete(server_coro)
    self.loop.run_until_complete(socks_auth())



# Copyright (C) 2017 Johnny Vestergaard <jkv@unixcluster.dk>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import csv
import logging
import json

from heralding.reporting.base_logger import BaseLogger

logger = logging.getLogger(__name__)


class FileLogger(BaseLogger):

  def __init__(self, session_csv_logfile, sessions_json_logfile, auth_logfile):
    super().__init__()

    self.auth_log_filehandler = None
    self.auth_log_writer = None
    self.session_csv_log_filehandler = None
    self.session_csv_log_writer = None
    self.session_json_log_filehandler = None

    if auth_logfile != "":
      # Setup CSV logging for auth attempts
      auth_field_names = [
          'timestamp', 'auth_id', 'session_id', 'source_ip', 'source_port',
          'destination_ip', 'destination_port', 'protocol', 'username',
          'password', 'password_hash'
      ]

      self.auth_log_filehandler, self.auth_log_writer = self.setup_csv_files(
          auth_logfile, auth_field_names)

      logger.info(
          'File logger: Using %s to log authentication attempts in CSV format.',
          auth_logfile)

    if session_csv_logfile != "":
      # Setup CSV logging for sessions
      session_field_names = [
          'timestamp', 'duration', 'session_id', 'source_ip', 'source_port',
          'destination_ip', 'destination_port', 'protocol', 'num_auth_attempts'
      ]
      self.session_csv_log_filehandler, self.session_csv_log_writer = self.setup_csv_files(
          session_csv_logfile, session_field_names)

      logger.info(
          'File logger: Using %s to log unified session data in CSV format.',
          session_csv_logfile)

    if sessions_json_logfile != "":
      # Setup json logging for logging complete sessions
      if not os.path.isfile(sessions_json_logfile):
        self.session_json_log_filehandler = open(
            sessions_json_logfile, 'w', encoding='utf-8')
      else:
        self.session_json_log_filehandler = open(
            sessions_json_logfile, 'a', encoding='utf-8')

      logger.info(
          'File logger: Using %s to log complete session data in JSON format.',
          sessions_json_logfile)

  def setup_csv_files(self, filename, field_names):
    handler = writer = None

    if not os.path.isfile(filename):
      handler = open(filename, 'w', encoding='utf-8')
    else:
      handler = open(filename, 'a', encoding='utf-8')

    writer = csv.DictWriter(
        handler, fieldnames=field_names, extrasaction='ignore')

    # empty file, write csv header
    if os.path.getsize(filename) == 0:
      writer.writeheader()
      handler.flush()

    return handler, writer

  def loggerStopped(self):
    for handler in [
        self.auth_log_filehandler, self.session_csv_log_filehandler,
        self.session_json_log_filehandler
    ]:
      if handler != None:
        handler.flush()
        handler.close()

  def handle_auth_log(self, data):
    # for now this logger only handles authentication attempts where we are able
    # to log both username and password
    if self.auth_log_filehandler != None:
      if 'username' in data and 'password' in data:
        self.auth_log_writer.writerow(data)
        # meh
        self.auth_log_filehandler.flush()

  def handle_session_log(self, data):
    if data['session_ended']:
      if self.session_csv_log_filehandler != None:
        self.session_csv_log_writer.writerow(data)
        self.session_csv_log_filehandler.flush()
      if self.session_json_log_filehandler != None:
        self.session_json_log_filehandler.write(json.dumps(data) + "\n")
        self.session_json_log_filehandler.flush()



# We need this in order to reduce the number of third-party modules.
# The main idea is to replace rfile/wfile with reader/writer and to
# add async/await syntax. So, it is http.server.BaseHTTPRequestHandler
# code, but adjusted to work with asyncio in our specific case.

import socket
import http.client

from http import HTTPStatus
from http.server import BaseHTTPRequestHandler

from .aioclient import parse_headers
from heralding.libs.aiobaserequest import AsyncBaseRequestHandler

# It is a hack in order to get the same class as BaseHTTPRequestHandler,
# but which inherits our AsyncBaseRequestHandler.
AsyncHttpHandler = type('AsyncHttpHandler', (AsyncBaseRequestHandler,),
                        dict(BaseHTTPRequestHandler.__dict__))


class AsyncBaseHTTPRequestHandler(AsyncHttpHandler):
  """Asynchronous analogue of http.server.BaseHTTPRequestHandler."""

  async def parse_request(self):
    self.command = None  # set in case of error on the first line
    self.request_version = version = self.default_request_version
    self.close_connection = True
    requestline = str(self.raw_requestline, 'iso-8859-1')
    requestline = requestline.rstrip('\r\n')
    self.requestline = requestline
    words = requestline.split()
    if len(words) == 3:
      command, path, version = words
      if version[:5] != 'HTTP/':
        self.send_error(HTTPStatus.BAD_REQUEST,
                        "Bad request version (%r)" % version)
        return False
      try:
        base_version_number = version.split('/', 1)[1]
        version_number = base_version_number.split(".")
        if len(version_number) != 2:
          raise ValueError
        version_number = int(version_number[0]), int(version_number[1])
      except (ValueError, IndexError):
        self.send_error(HTTPStatus.BAD_REQUEST,
                        "Bad request version (%r)" % version)
        return False
      if version_number >= (1, 1) and self.protocol_version >= "HTTP/1.1":
        self.close_connection = False
      if version_number >= (2, 0):
        self.send_error(HTTPStatus.HTTP_VERSION_NOT_SUPPORTED,
                        "Invalid HTTP Version (%s)" % base_version_number)
        return False
    elif len(words) == 2:
      command, path = words
      self.close_connection = True
      if command != 'GET':
        self.send_error(HTTPStatus.BAD_REQUEST,
                        "Bad HTTP/0.9 request type (%r)" % command)
        return False
    elif not words:
      return False
    else:
      self.send_error(HTTPStatus.BAD_REQUEST,
                      "Bad request syntax (%r)" % requestline)
      return False
    self.command, self.path, self.request_version = command, path, version

    # Examine the headers and look for a Connection directive.
    try:
      self.headers = await parse_headers(self.rfile, _class=self.MessageClass)
    except http.client.LineTooLong:
      self.send_error(HTTPStatus.BAD_REQUEST, "Line too long")
      return False
    except http.client.HTTPException as err:
      self.send_error(HTTPStatus.REQUEST_HEADER_FIELDS_TOO_LARGE,
                      "Too many headers", str(err))
      return False

    conntype = self.headers.get('Connection', "")
    if conntype.lower() == 'close':
      self.close_connection = True
    elif (conntype.lower() == 'keep-alive' and
          self.protocol_version >= "HTTP/1.1"):
      self.close_connection = False
    # Examine the headers and look for an Expect directive
    expect = self.headers.get('Expect', "")
    if (expect.lower() == "100-continue" and
        self.protocol_version >= "HTTP/1.1" and
        self.request_version >= "HTTP/1.1"):
      if not self.handle_expect_100():
        return False
    return True

  async def handle_one_request(self):
    try:
      self.raw_requestline = await self.rfile.readline()
      if len(self.raw_requestline) > 65536:
        self.requestline = ''
        self.request_version = ''
        self.command = ''
        self.send_error(HTTPStatus.REQUEST_URI_TOO_LONG)
        return
      if not self.raw_requestline:
        self.close_connection = True
        return
      if not await self.parse_request():
        # An error code has been sent, just exit
        return
      mname = 'do_' + self.command
      if not hasattr(self, mname):
        self.send_error(HTTPStatus.NOT_IMPLEMENTED,
                        "Unsupported method (%r)" % self.command)
        return
      method = getattr(self, mname)
      method()
    except socket.timeout as e:
      #a read or a write timed out.  Discard this connection
      self.log_error("Request timed out: %r", e)
      self.close_connection = True
      return

  async def handle(self):
    self.close_connection = True

    await self.handle_one_request()
    while not self.close_connection:
      await self.handle_one_request()



# Copyright (C) 2019 Sudipta Pandit <realsdx@protonmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import ssl
import logging

logger = logging.getLogger(__name__)


class TLSHandshakeError(Exception):

  def __init__(self, message=""):
    Exception.__init__(self, message)


class TLS:
  """ TLS implamentation using memory BIO """

  def __init__(self, writer, reader, pem_file):
    """@param: writer and reader are asyncio stream writer and reader objects"""
    self._tlsInBuff = ssl.MemoryBIO()
    self._tlsOutBuff = ssl.MemoryBIO()
    ctx = ssl.SSLContext(ssl.PROTOCOL_TLSv1_1)
    ctx.set_ciphers('RSA:!aNULL')
    ctx.check_hostname = False
    ctx.load_cert_chain(pem_file)
    self._tlsObj = ctx.wrap_bio(
        self._tlsInBuff, self._tlsOutBuff, server_side=True)
    self.writer = writer
    self.reader = reader

  async def do_tls_handshake(self):
    client_hello = await self.reader.read(4096)
    self._tlsInBuff.write(client_hello)
    try:
      self._tlsObj.do_handshake()
    except ssl.SSLError as e:
      if "WRONG_VERSION_NUMBER" in e.args[1]:
        logger.debug("Client tried to connect with wrong SSL version")
      else:
        logger.debug(e.args[1])
    except ssl.SSLWantReadError:
      server_hello = self._tlsOutBuff.read()
      self.writer.write(server_hello)
      await self.writer.drain()

    client_fin = await self.reader.read(4096)
    self._tlsInBuff.write(client_fin)
    try:
      self._tlsObj.do_handshake()
    except ssl.SSLWantReadError:
      raise TLSHandshakeError("Expected more data in Clinet FIN")

    server_fin = self._tlsOutBuff.read()
    self.writer.write(server_fin)
    await self.writer.drain()

  async def write_tls(self, data):
    self._tlsObj.write(data)
    _data = self._tlsOutBuff.read()
    _res = self.writer.write(_data)
    await self.writer.drain()
    return _res

  async def read_tls(self, size):
    data = b""
    # Check if we have any leftover data in the buffer
    try:
      data += self._tlsObj.read(size)
    except ssl.SSLWantReadError:
      pass

    # iterate until we have all needed plaintext
    while len(data) < size:
      if self.reader.at_eof():
        break
      try:
        # read ciphertext
        _rData = await self.reader.read(1)
        # put ciphertext into SSL machine
        self._tlsInBuff.write(_rData)
        # try to fill plaintext buffer
        data += self._tlsObj.read(size)
      except ssl.SSLWantReadError:
        pass

    return data



# Copyright (C) 2017 Johnny Vestergaard <jkv@unixcluster.dk>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import zmq
import logging

import heralding.misc
from heralding.misc.socket_names import SocketNames

logger = logging.getLogger(__name__)


class BaseLogger:

  def __init__(self):
    self.enabled = True

  def start(self):
    context = heralding.misc.zmq_context

    internal_reporting_socket = context.socket(zmq.SUB)
    internal_reporting_socket.connect(SocketNames.INTERNAL_REPORTING.value)
    internal_reporting_socket.setsockopt(zmq.SUBSCRIBE, b'')

    poller = zmq.Poller()
    poller.register(internal_reporting_socket, zmq.POLLIN)
    while self.enabled:
      socks = dict(poller.poll(500))
      self._execute_regulary()
      if internal_reporting_socket in socks and socks[
          internal_reporting_socket] == zmq.POLLIN:
        data = internal_reporting_socket.recv_pyobj()
        # if None is received, this means that ReportingRelay is going down
        if not data:
          self.stop()
        elif data['message_type'] == 'auth':
          self.handle_auth_log(data['content'])
        elif data['message_type'] == 'session_info':
          self.handle_session_log(data['content'])
        elif data['message_type'] == 'listen_ports':
          self.handle_listen_ports(data['content'])
        elif data['message_type'] == 'aux_info':
          self.handle_auxiliary_log(data['content'])
    internal_reporting_socket.close()
    # at this point we know no more data will arrive.
    self.loggerStopped()

  def stop(self):
    self.enabled = False

  def handle_auth_log(self, data):
    # should be handled in child class
    pass

  def handle_session_log(self, data):
    # implement if needed
    pass

  def handle_listen_ports(self, data):
    # implement if needed
    pass

  def handle_auxiliary_log(self, data):
    # implement if needed
    pass

  def _execute_regulary(self):
    # if implemented this method will get called regulary
    pass

  # called after we are sure no more data is received
  # override this to close filesockets, etc.
  def loggerStopped(self):
    pass



# We need this in order to reduce the number of third-party modules.
# It is a part of code from socketserver, that adjusted to work with
# asyncio in our specific case.


class AsyncBaseRequestHandler:
  """Asynchronous analogue of socketserver.BaseRequestHandler."""

  def __init__(self, reader, writer, client_address):
    self.rfile = reader
    self.wfile = writer
    self.client_address = client_address

  async def run(self):
    self.setup()
    try:
      await self.handle()
    finally:
      self.finish()

  def setup(self):
    pass

  async def handle(self):
    pass

  def finish(self):
    pass



# Copyright (C) 2019 Sudipta Pandit <realsdx@protonmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# *** This file contains all the PDU required for RDP Protocol ***
# each top tier PDU class will have a payload and generate method

from .packer import Uint16BE, Int16BE, Uint32LE, Uint32BE
from .parser import x224ConnectionRequestPDU
from .security import ServerSecurity


class tpktPDU:

  def __init__(self, payload):
    self.version = 3
    self.reserved = 0
    self.payload = payload

  def generate(self):
    payload_len = Int16BE.pack(1 + 1 + 2 + len(self.payload))
    # print(repr(payload_len))
    return bytes([self.version, self.reserved]) + payload_len + self.payload


class x224DataPDU:

  @staticmethod
  def generate():
    return b'\x02\xf0\x80'


class x224ConnectionConfirmPDU():

  def __init__(self, reqProto=None):
    self.type = b'\xd0'
    self.dest_ref = b'\x00\x00'  # 0 bytes
    self.src_ref = b'\x12\x34'  # bogus value
    self.options = b'\x00'
    self.reqProto = reqProto
    self.sentNegoFail = False

  def generate(self):
    PROTOCOL_SSL = 0x00000001

    data = self.type + self.dest_ref + self.src_ref + self.options
    len_indicator = len(data)  # 1byte length of PDU without indicator byte

    if self.reqProto:
      selectedProto = self.reqProto & PROTOCOL_SSL  # for now only tls
      proto_bytes = Uint32LE.pack(selectedProto)
      nego_res = b'\x02\x00\x08\x00' + proto_bytes

      # send NegoFail when a non-TLS method is requested
      if not (selectedProto == PROTOCOL_SSL):
        self.sentNegoFail = True
        nego_res = b'\x03\x00\x08\x00' + b'\x01' + bytes(
            3)  # SSL required by server
      len_indicator += len(nego_res)
      data += nego_res
    else:
      # RDP sec, if no proto requested. Currently not supported so send a Nego Fail
      self.sentNegoFail = True
      nego_res = b'\x03\x00\x08\x00' + b'\x01' + bytes(3)
      len_indicator += len(nego_res)
      data += nego_res

    return bytes([len_indicator]) + data

  def getFullPacket(self):
    return tpktPDU(self.generate()).generate()


class ServerData:

  @classmethod
  def generate(cls, cReqproto, serverSec):
    # Servr Core data
    coreData_1 = b'\x01\x0c\x0c\x00\x04\x00\x08\x00'
    clientReqPro = Uint32LE.pack(cReqproto)

    # ServerNetwork Data
    netData = b'\x03\x0c\x08\x00\xeb\x03\x00\x00'  # here channel count 0

    # ServerSec Data
    # here choosing encryption of 128bit by default method \x02\x00\x00\x00
    part_1 = b'\x02\x0c\xec\x00\x00\x00\x00\x00\x00\x00\x00\x00\x20\x00\x00\x00\xb8\x00\x00\x00'
    serverRandom = ServerSecurity.SERVER_RANDOM

    # an instance of ServerSecurity class required
    certData = serverSec.getServerCertBytes()
    # ADD all
    serverCoreData = coreData_1 + clientReqPro
    serverNetData = netData
    serverSecData = part_1 + serverRandom + certData

    return serverCoreData + serverNetData + serverSecData


class MCSConnectResponsePDU():
  """ Server MCS Connect Response PDU with GCC Conference Create Response """

  def __init__(self, cReqproto, serverSec):
    self.cReqproto = cReqproto
    self.serverSec = serverSec

  def generate(self):
    serverData = ServerData.generate(self.cReqproto, self.serverSec)
    serverDataLen = Uint16BE.pack(len(serverData) | 0x8000)
    gccCreateRes = b'\x00\x05\x00\x14\x7c\x00\x01\x2a\x14\x76\x0a\x01\x01\x00\x01\xc0\x00\x4d\x63\x44\x6e'
    cc_userDataLen = Uint16BE.pack(len(serverData) + 2 + len(gccCreateRes))
    cc_userData_1 = b'\x04\x82'
    domainParams = b'\x30\x1a\x02\x01\x22\x02\x01\x03\x02\x01\x00\x02\x01\x01\x02\x01\x00\x02\x01\x01\x02\x03\x00\xff\xf8\x02\x01\x02'
    cc_res = b'\x0a\x01\x00\x02\x01\x00'
    berLen = Uint16BE.pack(
        len(cc_res) + len(domainParams) + 2 + 2 + len(serverData) + 2 +
        len(gccCreateRes))
    bertype = b'\x7f\x66\x82'

    return bertype + berLen + cc_res + domainParams + cc_userData_1 + cc_userDataLen + gccCreateRes + serverDataLen + serverData

  def getFullPacket(self):
    return tpktPDU(x224DataPDU.generate() + self.generate()).generate()


class MCSAttachUserConfirmPDU():

  def generate(self):
    return b'\x2e\x00\x00\x06'

  # This is full static but just to be sillimar to other methods
  def getFullPacket(self):
    return tpktPDU(x224DataPDU.generate() + self.generate()).generate()


class MCSChannelJoinConfirmPDU():

  def __init__(self, initiator, channelID):
    self.initiator = Uint16BE.pack(initiator)
    self.channelID = Uint16BE.pack(channelID)

  def generate(self):
    res = b'\x3e\x00'
    return res + self.initiator + self.channelID + self.channelID

  def getFullPacket(self):
    return tpktPDU(x224DataPDU.generate() + self.generate()).generate()



# Copyright (C) 2017 Johnny Vestergaard <jkv@unixcluster.dk>
#
# Rewritten by Aniket Panse <contact@aniketpanse.in>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# Aniket Panse <contact@aniketpanse.in> grants Johnny Vestergaard <jkv@unixcluster.dk>
# a perpetual, worldwide, non-exclusive, no-charge, royalty-free, irrevocable
# copyright license to reproduce, prepare derivative works of, publicly
# display, publicly perform, sublicense, relicense, and distribute [the] Contributions
# and such derivative works.

import logging

from heralding.capabilities.handlerbase import HandlerBase

logger = logging.getLogger(__name__)

TERMINATOR = '\r\n'


class FtpHandler:
  """Handles a single FTP connection"""

  def __init__(self, reader, writer, options, session):
    self.banner = options['protocol_specific_data']['banner']
    self.max_loggins = int(options['protocol_specific_data']['max_attempts'])
    self.syst_type = options['protocol_specific_data']['syst_type']
    self.authenticated = False
    self.writer = writer
    self.reader = reader
    self.serve_flag = True
    self.session = session

    self.state = None
    self.user = None

  async def getcmd(self):
    cmd = await self.reader.readline()
    return str(cmd, 'utf-8')

  async def serve(self):
    await self.respond('220 ' + self.banner)

    while self.serve_flag:
      resp = await self.getcmd()
      if not resp:
        self.stop()
        break
      else:
        try:
          cmd, args = resp.split(' ', 1)
        except ValueError:
          cmd = resp
          args = None
        else:
          args = args.strip('\r\n')
        cmd = cmd.strip('\r\n')
        cmd = cmd.upper()
        # List of commands allowed before a login
        unauth_cmds = ['USER', 'PASS', 'QUIT', 'SYST']
        meth = getattr(self, 'do_' + cmd, None)
        if not meth:
          await self.respond('500 Unknown Command.')
        else:
          if not self.authenticated:
            if cmd not in unauth_cmds:
              await self.respond('503 Login with USER first.')
              continue
          await meth(args)
          self.state = cmd

  async def do_USER(self, arg):
    self.user = arg
    await self.respond('331 Now specify the Password.')

  async def do_PASS(self, arg):
    if self.state != 'USER':
      await self.respond('503 Login with USER first.')
      return
    passwd = arg
    self.session.add_auth_attempt(
        'plaintext', username=self.user, password=passwd)
    await self.respond('530 Authentication Failed.')
    if self.session.get_number_of_login_attempts() >= self.max_loggins:
      self.stop()

  async def do_SYST(self, arg):
    await self.respond('215 {0}'.format(self.syst_type))

  async def do_QUIT(self, arg):
    await self.respond('221 Bye.')
    self.serve_flag = False
    self.stop()

  async def respond(self, msg):
    msg += TERMINATOR
    msg_bytes = bytes(msg, 'utf-8')
    self.writer.write(msg_bytes)
    await self.writer.drain()

  def stop(self):
    self.session.end_session()


class ftp(HandlerBase):

  def __init__(self, options, loop):
    super().__init__(options, loop)
    self._options = options

  async def execute_capability(self, reader, writer, session):
    ftp_cap = FtpHandler(reader, writer, self._options, session)
    await ftp_cap.serve()



# Copyright (C) 2012 Aniket Panse <contact@aniketpanse.in
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# Aniket Panse <contact@aniketpanse.in> grants Johnny Vestergaard <jkv@unixcluster.dk>
# a perpetual, worldwide, non-exclusive, no-charge, royalty-free, irrevocable
# copyright license to reproduce, prepare derivative works of, publicly
# display, publicly perform, sublicense, relicense, and distribute [the] Contributions
# and such derivative works.

import hmac
import base64
import asyncio
import smtplib
import unittest

from heralding.capabilities import smtp
from heralding.reporting.reporting_relay import ReportingRelay


class SmtpTests(unittest.TestCase):

  def setUp(self):
    self.loop = asyncio.new_event_loop()
    asyncio.set_event_loop(None)

    self.reporting_relay = ReportingRelay()
    self.reporting_relay_task = self.loop.run_in_executor(
        None, self.reporting_relay.start)

  def tearDown(self):
    self.reporting_relay.stop()
    # We give reporting_relay a chance to be finished
    self.loop.run_until_complete(self.reporting_relay_task)

    self.server.close()
    self.loop.run_until_complete(self.server.wait_closed())

    self.loop.close()

  def test_connection(self):
    """ Tries to connect and run a EHLO command. Very basic test.
        """

    def smtp_connection():
      smtp_ = smtplib.SMTP(
          '127.0.0.1', 8888, local_hostname='localhost', timeout=15)
      smtp_.ehlo()
      smtp_.quit()

    # Use uncommon port so that we can run test even if the Honeypot is running.
    options = {
        'enabled': 'True',
        'port': 8888,
        'protocol_specific_data': {
            'banner': 'test'
        },
        'users': {
            'test': 'test'
        },
    }
    smtp_cap = smtp.smtp(options, self.loop)

    server_coro = asyncio.start_server(
        smtp_cap.handle_session, '0.0.0.0', 8888, loop=self.loop)
    self.server = self.loop.run_until_complete(server_coro)

    smtp_task = self.loop.run_in_executor(None, smtp_connection)
    self.loop.run_until_complete(smtp_task)

  def test_AUTH_CRAM_MD5_reject(self):
    """ Makes sure the server rejects all invalid login attempts that use the
            CRAM-MD5 Authentication method.
        """

    def encode_cram_md5(challenge, user, password):
      challenge = base64.decodebytes(challenge)
      response = user + b' ' + bytes(
          hmac.HMAC(password, challenge, digestmod="md5").hexdigest(), 'utf-8')
      return str(base64.b64encode(response), 'utf-8')

    def smtp_auth_cram_md5():
      smtp_ = smtplib.SMTP(
          '127.0.0.1', 8888, local_hostname='localhost', timeout=15)
      _, resp = smtp_.docmd('AUTH', 'CRAM-MD5')
      code, resp = smtp_.docmd(encode_cram_md5(resp, b'test', b'test'))
      smtp_.quit()
      # For now, the server's going to return a 535 code.
      self.assertEqual(code, 535)

    options = {
        'enabled': 'True',
        'port': 8888,
        'protocol_specific_data': {
            'banner': 'Test'
        },
        'users': {
            'someguy': 'test'
        }
    }
    smtp_cap = smtp.smtp(options, self.loop)

    server_coro = asyncio.start_server(
        smtp_cap.handle_session, '0.0.0.0', 8888, loop=self.loop)
    self.server = self.loop.run_until_complete(server_coro)

    smtp_task = self.loop.run_in_executor(None, smtp_auth_cram_md5)
    self.loop.run_until_complete(smtp_task)

  def test_AUTH_PLAIN_reject(self):
    """ Makes sure the server rejects all invalid login attempts that use the PLAIN Authentication method.
        """

    def smtp_auth_plain_reject():
      smtp_ = smtplib.SMTP(
          '127.0.0.1', 8888, local_hostname='localhost', timeout=15)
      arg = bytes('\0{0}\0{1}'.format('test', 'test'), 'utf-8')
      code, _ = smtp_.docmd('AUTH',
                            'PLAIN ' + str(base64.b64encode(arg), 'utf-8'))
      smtp_.quit()
      self.assertEqual(code, 535)

    options = {
        'enabled': 'True',
        'port': 0,
        'protocol_specific_data': {
            'banner': 'Test'
        },
        'users': {
            'someguy': 'test'
        }
    }

    smtp_cap = smtp.smtp(options, self.loop)

    server_coro = asyncio.start_server(
        smtp_cap.handle_session, '0.0.0.0', 8888, loop=self.loop)
    self.server = self.loop.run_until_complete(server_coro)

    smtp_task = self.loop.run_in_executor(None, smtp_auth_plain_reject)
    self.loop.run_until_complete(smtp_task)

  def test_AUTH_LOGIN_reject(self):
    """ Makes sure the server rejects all invalid login attempts that use the LOGIN Authentication method.
        """

    def smtp_auth_login_reject():
      smtp_ = smtplib.SMTP(
          '127.0.0.1', 8888, local_hostname='localhost', timeout=15)
      smtp_.docmd('AUTH', 'LOGIN')
      smtp_.docmd(str(base64.b64encode(b'test'), 'utf-8'))
      code, _ = smtp_.docmd(str(base64.b64encode(b'test'), 'utf-8'))
      smtp_.quit()
      self.assertEqual(code, 535)

    options = {
        'enabled': 'True',
        'port': 0,
        'protocol_specific_data': {
            'banner': 'Test'
        },
        'users': {
            'someguy': 'test'
        }
    }

    smtp_cap = smtp.smtp(options, self.loop)

    server_coro = asyncio.start_server(
        smtp_cap.handle_session, '0.0.0.0', 8888, loop=self.loop)
    self.server = self.loop.run_until_complete(server_coro)

    smtp_task = self.loop.run_in_executor(None, smtp_auth_login_reject)
    self.loop.run_until_complete(smtp_task)


if __name__ == '__main__':
  unittest.main()



# Copyright (C) 2017 Johnny Vestergaard <jkv@unixcluster.dk>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import unittest

from heralding.reporting.hpfeeds_logger import HpFeedsLogger


class FtpTests(unittest.TestCase):

  def test_hpfeeds(self):
    """Basic test for hpfeeds reporter"""

    session_channel = "heralding.session"
    auth_channel = "heraldign.auth"
    host = "127.0.0.1"
    port = 12345
    ident = "atzqøl"
    secret = "toosecret"

    HpFeedsLogger(session_channel, auth_channel, host, port, ident, secret)



# Copyright (C) 2019 Sudipta Pandit <realsdx@protonmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import struct
import logging

logger = logging.getLogger(__name__)


class InvalidExpectedData(Exception):

  def __init__(self, message=""):
    Exception.__init__(self, message)


class RawBytes():
  """ Read/Consume raw bytes """

  def __init__(self, data, structFormat, typeSize, pos=0, optional=False):
    self.data = data
    self._pos = pos
    self._structFormat = structFormat
    self._typeSize = typeSize
    self._optional = optional

  def dataLen(self):
    return len(self.data[self._pos:])

  def read(self):
    if self.dataLen() < self._typeSize:
      if self._optional:
        logger.debug("No optional data present in the PDU")
        return (b'', self._pos)
      else:
        raise InvalidExpectedData("Bytes Stream is too small to read")
    self.value = struct.unpack(
        self._structFormat, self.data[self._pos:self._pos + self._typeSize])[0]
    self._pos += self._typeSize

    return self.value, self._pos

  def readRaw(self):
    if self.dataLen() < self._typeSize:
      if self._optional:
        logger.debug("No optional data present in the PDU")
        return (b'', self._pos)
      else:
        raise InvalidExpectedData("Bytes Stream is too small to read")
    self.value = self.data[self._pos:self._pos + self._typeSize]
    self._pos += self._typeSize

    return self.value, self._pos

  def readUntil(self, until):
    if self.dataLen() < len(until) + 1:
      if self._optional:
        logger.debug("No optional data present in the PDU")
        return (b'', self._pos)
      else:
        raise InvalidExpectedData("Bytes Stream is too small to read")
    self.value = b''
    _data = self.data[self._pos:self._pos + len(until) + 1]
    while _data[-len(until):] != until:
      i = _data[0]
      self.value += i.to_bytes(1, byteorder='big')
      self._pos += 1
      _data = self.data[self._pos:self._pos + len(until) + 1]

    # insert the last char before mactching bytes
    i = _data[0]
    self.value += i.to_bytes(1, byteorder='big')
    self._pos += 1

    self._pos += len(until)
    return self.value, self._pos


class UInt8(RawBytes):

  def __init__(self, data, pos, optional=False):
    RawBytes.__init__(self, data, "B", 1, pos, optional)


class SInt8(RawBytes):

  def __init__(self, data, pos, optional=False):
    RawBytes.__init__(self, data, "b", 1, pos, optional)


class UInt16Be(RawBytes):

  def __init__(self, data, pos, optional=False):
    RawBytes.__init__(self, data, ">H", 2, pos, optional)


class UInt16Le(RawBytes):

  def __init__(self, data, pos, optional=False):
    RawBytes.__init__(self, data, "<H", 2, pos, optional)


class UInt32Be(RawBytes):

  def __init__(self, data, pos, optional=False):
    RawBytes.__init__(self, data, ">I", 4, pos, optional)


class UInt32Le(RawBytes):

  def __init__(self, data, pos, optional=False):
    RawBytes.__init__(self, data, "<I", 4, pos, optional)


class tpktPDUParser():

  def __init__(self):
    # only length can be uselful
    self.length = None

  def parse(self, raw_data, pos=0):
    """Returs pos of the rest of the payload"""
    _, pos = RawBytes(raw_data, None, 2,
                      pos).readRaw()  # consume version and reserved
    self.length, pos = UInt16Be(raw_data, pos).read()
    return pos


class x224DataPDU():

  @classmethod
  def parse(cls, raw_data, pos):
    """Returns the pos of the rest of the Payload"""
    _, pos = RawBytes(raw_data, None, 3, pos).readRaw()
    return pos


class x224ConnectionRequestPDU():
  # length            1byte
  # pdutype/credit    1byte
  # dst_ref           2byte
  # src_ref           2byte
  # options           1byte

  def __init__(self):
    # only pdu type is required
    self.pduType = None
    self.cookie = None
    self.reqProtocols = False

  def parse(self, raw_data, pos=0):
    pos = tpktPDUParser().parse(raw_data, 0)
    self.pduType, pos = UInt8(raw_data, pos + 1).read()  # ignore lenght byte
    _, pos = RawBytes(raw_data, None, 5, pos).readRaw()  # consume 5bytes
    # cookie is optional
    if b"Cookie: mstshash=" in raw_data:
      self.cookie, pos = RawBytes(raw_data, None, None,
                                  pos).readUntil(b"\x0d\x0a")
    # parse nego req if present
    if raw_data[pos:] != b'':
      _, pos = RawBytes(raw_data, None, 4, pos).readRaw()
      self.reqProtocols, pos = UInt32Le(raw_data, pos).read()

    return pos


class MCSChannelJoinRequestPDU():

  def __init__(self):
    self.header = None
    self.initiator = None
    self.channelID = None

  def parse(self, raw_data, pos=0):
    pos = tpktPDUParser().parse(raw_data, 0)
    pos = x224DataPDU().parse(raw_data, pos)
    self.header, pos = RawBytes(raw_data, None, 1, pos).readRaw()
    if self.header != b'\x38':
      return -1

    self.initiator, pos = UInt16Be(raw_data, pos).read()
    self.channelID, pso = UInt16Be(raw_data, pos).read()
    return pos


class ErectDomainRequestPDU():

  @staticmethod
  def checkPDU(raw_data, pos=0):
    # Type constant of ErectDomainRequest
    ERECT_DOMAIN_REQUEST = 1

    pos = tpktPDUParser().parse(raw_data, 0)
    pos = x224DataPDU().parse(raw_data, pos)
    pdu_type, pos = UInt8(raw_data, pos).read()

    if pdu_type == (ERECT_DOMAIN_REQUEST << 2):
      return True

    return False


class AttachUserRequestPDU():

  @staticmethod
  def checkPDU(raw_data, pos=0):
    # Type constant of AttachUserRequest
    ATTACH_USER_REQUEST = 10

    pos = tpktPDUParser().parse(raw_data, 0)
    pos = x224DataPDU().parse(raw_data, pos)
    pdu_type, pos = UInt8(raw_data, pos).read()

    if pdu_type == (ATTACH_USER_REQUEST << 2):
      return True

    return False


class ClientSecurityExcahngePDU():

  def __init__(self):
    self.secHeaderFlags = None
    self.secPacketLen = None
    self.encClientRandom = None

  def parse(self, raw_data, pos=0):
    pos = tpktPDUParser().parse(raw_data, 0)
    pos = x224DataPDU().parse(raw_data, pos)
    _, pos = RawBytes(raw_data, None, 8, pos).readRaw()  # 7 changed to 8
    self.secHeaderFlags, pos = UInt16Le(raw_data, pos).read()
    # +2 for skipkking bytes read
    self.secPacketLen, pos = UInt32Le(raw_data, pos + 2).read()
    # not reading last 8byte padding
    self.encClientRandom, pos = RawBytes(raw_data, None, self.secPacketLen - 8,
                                         pos).readRaw()
    return pos


class ClientInfoPDU():

  def __init__(self):
    self.secHeaderFlags = None
    self.infoLen = None
    self.dataSig = None
    self.encData = None

    # from decrypted data
    self.rdpUsername = None
    self.rdpPassword = None

  def parseTLS(self, raw_data, pos=0):
    pos = tpktPDUParser().parse(raw_data, 0)
    pos = x224DataPDU().parse(raw_data, pos)
    _, pos = RawBytes(raw_data, None, 6, pos).readRaw()
    # read length bytes (PER encoded)
    _infoLen, pos = UInt16Be(raw_data, pos).read()
    self.infoLen = _infoLen & 0x0fff
    # consume flags(2), flagsHi(2), CodePage(4), OptionalFlags(4)
    _, pos = RawBytes(raw_data, None, 12, pos).readRaw()
    #  cbParams(2+2+2+2+2)
    cbDomain, pos = UInt16Le(raw_data, pos).read()
    cbUsername, pos = UInt16Le(raw_data, pos).read()
    cbPassword, pos = UInt16Le(raw_data, pos).read()
    cbAltShell, pos = UInt16Le(raw_data, pos).read()
    cbWorkDir, pos = UInt16Le(raw_data, pos).read()
    # mandatory NULL terminator for all cbPrams is 2 bytes
    Domain, pos = RawBytes(raw_data, None, cbDomain + 2, pos).readRaw()
    Username, pos = RawBytes(raw_data, None, cbUsername + 2, pos).readRaw()
    Password, pos = RawBytes(raw_data, None, cbPassword + 2, pos).readRaw()
    AltShell, pos = RawBytes(raw_data, None, cbAltShell + 2, pos).readRaw()
    WorkDir, pos = RawBytes(raw_data, None, cbWorkDir + 2, pos).readRaw()

    # strip the last two null bytes
    self.rdpUsername = Username.decode('utf-16', 'ignore')[:-1]
    self.rdpPassword = Password.decode('utf-16', 'ignore')[:-1]
    return pos



# Copyright (C) 2017 Roman Samoilenko <ttahabatt@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import logging
import functools

from heralding.capabilities.handlerbase import HandlerBase

import asyncssh
from Crypto.PublicKey import RSA

logger = logging.getLogger(__name__)


class SSH(asyncssh.SSHServer, HandlerBase):
  connections_list = []

  def __init__(self, options, loop):
    asyncssh.SSHServer.__init__(self)
    HandlerBase.__init__(self, options, loop)

  def connection_made(self, conn):
    SSH.connections_list.append(conn)
    self.address = conn.get_extra_info('peername')
    self.dest_address = conn.get_extra_info('sockname')
    self.connection = conn
    self.handle_connection()
    logger.debug('SSH connection received from %s.' %
                 conn.get_extra_info('peername')[0])

  def connection_lost(self, exc):
    self.session.set_auxiliary_data(self.get_auxiliary_data())
    self.close_session(self.session)
    if exc:
      logger.debug('SSH connection error: ' + str(exc))
    else:
      logger.debug('SSH connection closed.')

  def begin_auth(self, username):
    return True

  def password_auth_supported(self):
    return True

  def validate_password(self, username, password):
    self.session.add_auth_attempt(
        'plaintext', username=username, password=password)
    return False

  def handle_connection(self):
    if HandlerBase.global_sessions > HandlerBase.MAX_GLOBAL_SESSIONS:
      protocol = self.__class__.__name__.lower()
      logger.warning(
          'Got {0} session on port {1} from {2}:{3}, but not handling it because the global session limit has '
          'been reached'.format(protocol, self.port, *self.address))
    else:
      self.session = self.create_session(self.address, self.dest_address)

  def get_auxiliary_data(self):
    data_fields = [
        'client_version', 'recv_cipher', 'recv_mac', 'recv_compression'
    ]
    data = {f: self.connection.get_extra_info(f) for f in data_fields}
    return data

  @staticmethod
  def change_server_banner(banner):
    """_send_version code was copied from asyncssh.connection in order to change
        internal local variable 'version', providing custom banner."""

    @functools.wraps(asyncssh.connection.SSHConnection._send_version)
    def _send_version(self):
      """Start the SSH handshake"""

      version = bytes(banner, 'utf-8')

      if self.is_client():
        self._client_version = version
        self._extra.update(client_version=version.decode('ascii'))
      else:
        self._server_version = version
        self._extra.update(server_version=version.decode('ascii'))

      self._send(version + b'\r\n')

    asyncssh.connection.SSHConnection._send_version = _send_version

  @staticmethod
  def generate_ssh_key(ssh_key_file):
    if not os.path.isfile(ssh_key_file):
      with open(ssh_key_file, 'w') as _file:
        rsa_key = RSA.generate(2048)
        priv_key_text = str(rsa_key.exportKey('PEM', pkcs=1), 'utf-8')
        _file.write(priv_key_text)



# Copyright (C) 2017 Johnny Vestergaard <jkv@unixcluster.dk>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import asyncio
import unittest

import ftplib
from ftplib import FTP

from heralding.capabilities import ftp
from heralding.reporting.reporting_relay import ReportingRelay


class FtpTests(unittest.TestCase):

  def setUp(self):
    self.loop = asyncio.new_event_loop()
    asyncio.set_event_loop(None)

    self.reporting_relay = ReportingRelay()
    self.reporting_relay_task = self.loop.run_in_executor(
        None, self.reporting_relay.start)

  def tearDown(self):
    self.reporting_relay.stop()
    # We give reporting_relay a chance to be finished
    self.loop.run_until_complete(self.reporting_relay_task)

    self.server.close()
    self.loop.run_until_complete(self.server.wait_closed())

    self.loop.close()

  def test_login(self):
    """Testing different login combinations"""

    def ftp_login():
      ftp_client = FTP()
      ftp_client.connect('127.0.0.1', 8888, 1)
      # expect perm exception
      try:
        ftp_client.login('james', 'bond')
        _ = ftp_client.getresp()  # NOQA
      except ftplib.error_perm:
        ftp_client.quit()

    options = {
        'enabled': 'True',
        'port': 0,
        'banner': 'Test Banner',
        'users': {
            'test': 'test'
        },
        'protocol_specific_data': {
            'max_attempts': 3,
            'banner': 'test banner',
            'syst_type': 'Test Type'
        }
    }

    ftp_capability = ftp.ftp(options, self.loop)

    server_coro = asyncio.start_server(
        ftp_capability.handle_session, '0.0.0.0', 8888, loop=self.loop)
    self.server = self.loop.run_until_complete(server_coro)

    ftp_task = self.loop.run_in_executor(None, ftp_login)
    self.loop.run_until_complete(ftp_task)



import struct
import logging

from heralding.capabilities.handlerbase import HandlerBase

logger = logging.getLogger(__name__)


class PostgreSQL(HandlerBase):

  async def execute_capability(self, reader, writer, session):
    try:
      await self._handle_session(session, reader, writer)
    except struct.error as exc:
      logger.debug('PostgreSQL connection error: %s', exc)
      session.end_session()

  async def _handle_session(self, session, reader, writer):
    # Read (we blindly assume) SSL request and deny it
    await read_msg(reader, writer)

    writer.write(b'N')

    session.activity()

    # Read login details
    data = await read_msg(reader, writer)
    login_dict = parse_dict(data)

    # Request plain text password login
    password_request = [b'R', 8, 3]
    writer.write(struct.pack('>c I I', *password_request))
    await writer.drain()

    # Read password
    data = await read_msg(reader, writer)
    password = parse_str(data)
    username = login_dict['user']
    session.add_auth_attempt('plaintext', username=username, password=password)

    # Report login failure
    writer.write(b'E')
    fail = [
        b'SFATAL\x00C28P01\x00',
        'Mpassword authentication failed for user "{}"'.format(username).encode(
            'utf-8'),
        b'\x00Fauth.c\x00L288\x00Rauth_failed\x00\x00',
    ]

    length = sum([len(f) for f in fail])
    writer.write(struct.pack('>I', length + 4))
    for f in fail:
      writer.write(f)

    await writer.drain()

    session.end_session()


async def read_msg(reader, writer):
  i = await reader.read(4)
  length = struct.unpack('>I', i)[0]
  data = await reader.read(length)
  return data


def parse_dict(data):
  dct = {}
  mode = 'pad'
  key = []
  value = []

  for c in struct.iter_unpack('c', data):
    c = c[0]

    if mode == 'pad':
      if c in (bytes([0]), bytes([3])):
        continue
      else:
        mode = 'key'

    if mode == 'key':
      if c == bytes([0]):
        mode = 'value'
      else:
        key.append(c.decode())

    elif mode == 'value':
      if c == bytes([0]):
        dct[''.join(key)] = ''.join(value)
        key = []
        value = []
        mode = 'pad'
      else:
        value.append(c.decode())

  return dct


def parse_str(data):
  data_array = bytearray(data)
  return data_array[1:-1].decode('utf-8')



# Copyright (C) 2017 Johnny Vestergaard <jkv@unixcluster.dk>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import ssl
import logging
import asyncio

import heralding.misc.common as common
import heralding.capabilities.handlerbase

from heralding.reporting.reporting_relay import ReportingRelay
from heralding.reporting.file_logger import FileLogger
from heralding.reporting.syslog_logger import SyslogLogger
from heralding.reporting.hpfeeds_logger import HpFeedsLogger
from heralding.reporting.curiosum_integration import CuriosumIntegration

import asyncssh

logger = logging.getLogger(__name__)


class Honeypot:
  public_ip = ''
  wordlist = None

  def __init__(self, config, loop):
    """
        :param config: configuration dictionary.
        """
    assert config is not None
    self.loop = loop
    self.SshClass = None
    self.config = config
    self._servers = []
    self._loggers = []

  async def _record_and_lookup_public_ip(self):
    while True:
      try:
        Honeypot.public_ip = common.get_public_ip()
        logger.warning('Found public ip: %s', Honeypot.public_ip)
      except Exception as ex:
        Honeypot.public_ip = ''
        logger.warning('Could not request public ip from ipify, error: %s', ex)
      await asyncio.sleep(3600)

  def setup_wordlist(self):
    # load wordlist in memory
    wordlist_file = self.config['hash_cracker']['wordlist_file']
    if not os.path.isfile(wordlist_file):
      package_directory = os.path.dirname(os.path.abspath(heralding.__file__))
      wordlist_file = os.path.join(package_directory, wordlist_file)
      logger.warning(
          'Using default wordlist file: "{0}", if you want to customize values please '
          'copy this file to the current working directory'.format(
              wordlist_file))
    with open(wordlist_file, 'r') as f:
      Honeypot.wordlist = f.read().splitlines()

  def start(self):
    """ Starts services. """

    if 'public_ip_as_destination_ip' in self.config and self.config[
        'public_ip_as_destination_ip'] is True:
      asyncio.ensure_future(self._record_and_lookup_public_ip(), loop=self.loop)

    # setup hash cracker's wordlist
    if self.config['hash_cracker']['enabled']:
      self.setup_wordlist()

    # start activity logging
    if 'activity_logging' in self.config:
      if 'file' in self.config['activity_logging'] and self.config[
          'activity_logging']['file']['enabled']:
        auth_log = self.config['activity_logging']['file'][
            'authentication_log_file']
        session_csv_log = self.config['activity_logging']['file'][
            'session_csv_log_file']
        session_json_log = self.config['activity_logging']['file'][
            'session_json_log_file']
        file_logger = FileLogger(session_csv_log, session_json_log, auth_log)
        self.file_logger_task = self.loop.run_in_executor(
            None, file_logger.start)
        self.file_logger_task.add_done_callback(
            common.on_unhandled_task_exception)
        self._loggers.append(file_logger)

      if 'syslog' in self.config['activity_logging'] and self.config[
          'activity_logging']['syslog']['enabled']:
        sys_logger = SyslogLogger()
        self.sys_logger_task = self.loop.run_in_executor(None, sys_logger.start)
        self.sys_logger_task.add_done_callback(
            common.on_unhandled_task_exception)
        self._loggers.append(sys_logger)

      if 'hpfeeds' in self.config['activity_logging'] and self.config[
          'activity_logging']['hpfeeds']['enabled']:
        session_channel = self.config['activity_logging']['hpfeeds'][
            'session_channel']
        auth_channel = self.config['activity_logging']['hpfeeds'][
            'auth_channel']
        host = self.config['activity_logging']['hpfeeds']['host']
        port = self.config['activity_logging']['hpfeeds']['port']
        ident = self.config['activity_logging']['hpfeeds']['ident']
        secret = self.config['activity_logging']['hpfeeds']['secret']
        hpfeeds_logger = HpFeedsLogger(session_channel, auth_channel, host,
                                       port, ident, secret)
        self.hpfeeds_logger_task = self.loop.run_in_executor(
            None, hpfeeds_logger.start)
        self.hpfeeds_logger_task.add_done_callback(
            common.on_unhandled_task_exception)

      if 'curiosum' in self.config['activity_logging'] and self.config[
          'activity_logging']['curiosum']['enabled']:
        port = self.config['activity_logging']['curiosum']['port']
        curiosum_integration = CuriosumIntegration(port)
        self.hpfeeds_logger_task = self.loop.run_in_executor(
            None, curiosum_integration.start)
        self.hpfeeds_logger_task.add_done_callback(
            common.on_unhandled_task_exception)

    bind_host = self.config['bind_host']
    listen_ports = []
    for c in heralding.capabilities.handlerbase.HandlerBase.__subclasses__():
      cap_name = c.__name__.lower()
      if cap_name in self.config['capabilities']:
        if not self.config['capabilities'][cap_name]['enabled']:
          continue
        port = self.config['capabilities'][cap_name]['port']
        listen_ports.append(port)
        # carve out the options for this specific service
        options = self.config['capabilities'][cap_name]
        # capabilities are only allowed to append to the session list
        cap = c(options, self.loop)
        try:
          # # Convention: All capability names which end in 's' will be wrapped in ssl.
          if cap_name.endswith('s'):
            pem_file = '{0}.pem'.format(cap_name)
            self.create_cert_if_not_exists(cap_name, pem_file)
            ssl_context = self.create_ssl_context(pem_file)
            server_coro = asyncio.start_server(
                cap.handle_session,
                bind_host,
                port,
                loop=self.loop,
                ssl=ssl_context)
          elif cap_name == 'ssh':
            # Since dicts and user-defined classes are mutable, we have
            # to save ssh class and ssh options somewhere.
            ssh_options = options
            SshClass = c
            self.SshClass = SshClass

            ssh_key_file = 'ssh.key'
            SshClass.generate_ssh_key(ssh_key_file)

            banner = ssh_options['protocol_specific_data']['banner']
            SshClass.change_server_banner(banner)

            server_coro = asyncssh.create_server(
                lambda: SshClass(ssh_options, self.loop),
                bind_host,
                port,
                server_host_keys=[ssh_key_file],
                login_timeout=cap.timeout)
          elif cap_name == 'rdp':
            pem_file = '{0}.pem'.format(cap_name)
            self.create_cert_if_not_exists(cap_name, pem_file)
            server_coro = asyncio.start_server(
                cap.handle_session, bind_host, port, loop=self.loop)
          else:
            server_coro = asyncio.start_server(
                cap.handle_session, bind_host, port, loop=self.loop)

          server = self.loop.run_until_complete(server_coro)
          logger.debug('Adding %s capability with options: %s', cap_name,
                       options)
          self._servers.append(server)
        except Exception as ex:
          error_message = "Could not start {0} server on port {1}. Error: {2}".format(
              c.__name__, port, ex)
          logger.error(error_message)
          raise ex
        else:
          logger.info('Started %s capability listening on port %s', c.__name__,
                      port)
    ReportingRelay.logListenPorts(listen_ports)

  def stop(self):
    """Stops services"""
    if self.config['capabilities']['ssh']['enabled'] and self.SshClass != None:
      for conn in self.SshClass.connections_list:
        conn.close()
        self.loop.run_until_complete(conn.wait_closed())

    for server in self._servers:
      server.close()
      self.loop.run_until_complete(server.wait_closed())

    for l in self._loggers:
      l.stop()

    self.loop.run_until_complete(common.cancel_all_pending_tasks(self.loop))

    logger.info('All tasks were stopped.')

  def create_cert_if_not_exists(self, cap_name, pem_file):
    if not os.path.isfile(pem_file):
      logger.debug('Generating certificate and key: %s', pem_file)

      cert_dict = self.config['capabilities'][cap_name][
          'protocol_specific_data']['cert']
      cert_cn = cert_dict['common_name']
      cert_country = cert_dict['country']
      cert_state = cert_dict['state']
      cert_locality = cert_dict['locality']
      cert_org = cert_dict['organization']
      cert_org_unit = cert_dict['organizational_unit']
      valid_days = int(cert_dict['valid_days'])
      serial_number = int(cert_dict['serial_number'])

      cert, key = common.generate_self_signed_cert(cert_country, cert_state,
                                                   cert_org, cert_locality,
                                                   cert_org_unit, cert_cn,
                                                   valid_days, serial_number)
      with open(pem_file, 'wb') as _pem_file:
        _pem_file.write(cert)
        _pem_file.write(key)

  @staticmethod
  def create_ssl_context(pem_file):
    ssl_context = ssl.create_default_context(ssl.Purpose.CLIENT_AUTH)
    ssl_context.check_hostname = False
    ssl_context.load_cert_chain(pem_file)
    return ssl_context



import asyncio
import unittest

import pymysql
from heralding.capabilities import mysql
from heralding.misc.common import cancel_all_pending_tasks
from heralding.reporting.reporting_relay import ReportingRelay


class MySQLTests(unittest.TestCase):

  def setUp(self):
    self.loop = asyncio.new_event_loop()
    asyncio.set_event_loop(None)

    self.reporting_relay = ReportingRelay()
    self.reporting_relay_task = self.loop.run_in_executor(
        None, self.reporting_relay.start)

  def tearDown(self):
    self.reporting_relay.stop()
    # We give reporting_relay a chance to be finished
    self.loop.run_until_complete(self.reporting_relay_task)

    self.server.close()
    self.loop.run_until_complete(self.server.wait_closed())

    self.loop.run_until_complete(cancel_all_pending_tasks(self.loop))
    self.loop.close()

  def test_invalid_login(self):
    """Tests if mysql server responds correctly to a invalid login attempt."""

    def mysql_login():
      try:
        pymysql.connect(
            host="0.0.0.0",
            port=8306,
            user="tuser",
            password="tpass",
            db="testdb")
      except pymysql.err.OperationalError as e:
        return e
      return None

    options = {'enabled': 'True', 'port': 8306}
    mysql_cap = mysql.MySQL(options, self.loop)

    server_coro = asyncio.start_server(
        mysql_cap.handle_session, '0.0.0.0', 8306, loop=self.loop)
    self.server = self.loop.run_until_complete(server_coro)

    mysql_task = self.loop.run_in_executor(None, mysql_login)
    login_exception = self.loop.run_until_complete(mysql_task)

    self.assertIsInstance(login_exception, pymysql.err.OperationalError)
    self.assertEqual(
        str(login_exception),
        '(1045, "Access denied for user \'tuser\'@\'127.0.0.1\' (using password: YES)")'
    )

  def cb(self, socket, command, option):
    return



# Copyright (C) 2017 Johnny Vestergaard <jkv@unixcluster.dk>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import logging
import asyncio
import requests

from OpenSSL import crypto
from Crypto.PublicKey import RSA

logger = logging.getLogger(__name__)


def on_unhandled_task_exception(task):
  if not task.cancelled():
    task_exc = task.exception()
    if task_exc:
      logger.exception('Stopping because %s died: %s', task, task_exc)
      os._exit(1)


async def cancel_all_pending_tasks(loop=None):
  if loop is None:
    loop = asyncio.get_event_loop()
  pending = asyncio.Task.all_tasks(loop=loop)
  pending.remove(asyncio.Task.current_task(loop=loop))
  for task in pending:
    # We give task only 1 second to die.
    if not task.done():
      task.cancel()
      try:
        await asyncio.wait_for(task, timeout=5, loop=loop)
      except (asyncio.CancelledError, KeyboardInterrupt, ConnectionResetError):
        pass


def generate_self_signed_cert(cert_country, cert_state, cert_organization,
                              cert_locality, cert_organizational_unit,
                              cert_common_name, valid_days, serial_number):
  rsa_key = RSA.generate(2048)

  pk = crypto.load_privatekey(crypto.FILETYPE_PEM,
                              rsa_key.exportKey('PEM', pkcs=1))
  cert = crypto.X509()
  sub = cert.get_subject()
  sub.CN = cert_common_name
  sub.C = cert_country
  sub.ST = cert_state
  sub.L = cert_locality
  sub.O = cert_organization

  # optional
  if cert_organizational_unit:
    sub.OU = cert_organizational_unit

  cert.set_serial_number(serial_number)
  cert.gmtime_adj_notBefore(0)
  cert.gmtime_adj_notAfter(valid_days * 24 * 60 * 60)  # Valid for a year
  cert.set_issuer(sub)
  cert.set_pubkey(pk)
  cert.sign(pk, 'sha1')

  cert_text = crypto.dump_certificate(crypto.FILETYPE_PEM, cert)
  priv_key_text = rsa_key.exportKey('PEM', pkcs=1)

  return cert_text, priv_key_text


def get_public_ip():
  r = requests.get('https://api.ipify.org')
  r.raise_for_status()
  return r.text



#
# Copyright (c) 2014-2015 Sylvain Peyrefitte
#
# Some contents of this file is part of rdpy.
#
# rdpy is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#

import rsa
import hashlib

# Generate New RSA Keys
rsa_key = rsa.newkeys(512)


def getRSAKeys():
  """Returns a 512-Bit RSA keys"""
  return rsa_key


def PrivateKey(d, n):
  """
    @param d: {long | str}private exponent
    @param n: {long | str}modulus
    """
  if isinstance(d, bytes):
    d = rsa.transform.bytes2int(d)
  if isinstance(n, bytes):
    n = rsa.transform.bytes2int(n)
  return {'d': d, 'n': n}


def int2bytes(i, fill_size=0):
  """wrapper of rsa.transform.int2bytes"""
  return rsa.transform.int2bytes(i, fill_size)


def signRSA(message, privateKey):
  """
    @summary: sign message with private key
    @param message: {str} message to sign
    @param privateKey : {rsa.privateKey} key use to sugn
    """
  return rsa.transform.int2bytes(
      rsa.core.encrypt_int(
          rsa.transform.bytes2int(message), privateKey['d'], privateKey['n']),
      rsa.common.byte_size(privateKey['n']))


def decryptRSA(message, privateKey):
  """
    @summary: wrapper around rsa.core.decrypt_int function
    @param message: {str} source message
    @param publicKey: {rsa.PrivateKey}
    """
  return rsa.transform.int2bytes(
      rsa.core.decrypt_int(
          rsa.transform.bytes2int(message), privateKey['d'], privateKey['n']))


class ServerSecurity():
  # http://msdn.microsoft.com/en-us/library/cc240776.aspx
  _TERMINAL_SERVICES_MODULUS_ = b"\x3d\x3a\x5e\xbd\x72\x43\x3e\xc9\x4d\xbb\xc1\x1e\x4a\xba\x5f\xcb\x3e\x88\x20\x87\xef\xf5\xc1\xe2\xd7\xb7\x6b\x9a\xf2\x52\x45\x95\xce\x63\x65\x6b\x58\x3a\xfe\xef\x7c\xe7\xbf\xfe\x3d\xf6\x5c\x7d\x6c\x5e\x06\x09\x1a\xf5\x61\xbb\x20\x93\x09\x5f\x05\x6d\xea\x87"
  _TERMINAL_SERVICES_PRIVATE_EXPONENT_ = b"\x87\xa7\x19\x32\xda\x11\x87\x55\x58\x00\x16\x16\x25\x65\x68\xf8\x24\x3e\xe6\xfa\xe9\x67\x49\x94\xcf\x92\xcc\x33\x99\xe8\x08\x60\x17\x9a\x12\x9f\x24\xdd\xb1\x24\x99\xc7\x3a\xb8\x0a\x7b\x0d\xdd\x35\x07\x79\x17\x0b\x51\x9b\xb3\xc7\x10\x01\x13\xe7\x3f\xf3\x5f"
  _TERMINAL_SERVICES_PUBLIC_EXPONENT_ = b"\x5b\x7b\x88\xc0"

  SERVER_RANDOM = b'\xf9\xdf\xc4p\x8a\x08\xcds5Q:\xa3!_\x8f\xa1\xeeQ\xd6Nj\x95Zy0\x12\xbf\xf5&\xfb!('
  SERVER_PUBKEY_PROPS_1 = b'\x01\x00\x00\x00\x01\x00\x00\x00\x01\x00\x00\x00\x06\x00\x5c\x00'
  SERVER_PUBKEY_PROPS_2 = b'RSA1H\x00\x00\x00\x00\x02\x00\x00?\x00\x00\x00'
  PADDING = b'\x00\x00\x00\x00\x00\x00\x00\x00'
  SERVER_PUBLIC_EXPONENT = b'\x01\x00\x01\x00'

  # SERVER_MODULUS = b'g3\x9c\xd5\x94\xa0\tO\xbfrVt`\xab\x7f\xf1+\xf7J\x86\xa3(ZK\x12\xc44(\xcd\xec"E6a\xf3\x1d&\t\x8fL\xb6\xc8f\x9c\xd0\xa3\xaf8\xaa\xe1\xfb\xcb\\\r\xfc\xbb\xd8\x93\x1cZ\xce\x1b\x8f\x92\x00\x00\x00\x00\x00\x00\x00\x00'

  def __init__(self):
    self._pubKey, self._privKey = getRSAKeys()
    self._modulusBytes = int2bytes(self._pubKey.n)
    # self._exponentBytes = int2bytes(self._pubKey.e)
    self._exponentBytes = ServerSecurity.SERVER_PUBLIC_EXPONENT
    self._clientRandom = None  # set it

  def getSignatureHash(self):
    # this is a signature of first 6 fields in ServerCert
    DATA = ServerSecurity.SERVER_PUBKEY_PROPS_1+ServerSecurity.SERVER_PUBKEY_PROPS_2 + \
        self._exponentBytes+self._modulusBytes+ServerSecurity.PADDING

    md5Digest = hashlib.md5()
    md5Digest.update(DATA)
    return md5Digest.digest() + b"\x00" + b"\xff" * 45 + b"\x01"

  def getServerCertBytes(self):
    sigHash = signRSA(
        self.getSignatureHash()[::-1],
        PrivateKey(
            d=ServerSecurity._TERMINAL_SERVICES_PRIVATE_EXPONENT_[::-1],
            n=ServerSecurity._TERMINAL_SERVICES_MODULUS_[::-1]))[::-1]
    sigBlobProps = b'\x08\x00\x48\x00'
    return ServerSecurity.SERVER_PUBKEY_PROPS_1+ServerSecurity.SERVER_PUBKEY_PROPS_2 + \
        self._exponentBytes+self._modulusBytes+ServerSecurity.PADDING+sigBlobProps+sigHash+ServerSecurity.PADDING

  def decryptClientRandom(self, encRandom):
    self._decClientRandom = decryptRSA(encRandom[::-1], self._privKey)[::-1]
    return self._decClientRandom



import asyncio
import unittest
import socket
import ssl
import os

from heralding.capabilities import rdp
from heralding.misc.common import cancel_all_pending_tasks, generate_self_signed_cert
from heralding.reporting.reporting_relay import ReportingRelay


class RDPClient():

  @classmethod
  def ConnectionRequestPDU(cls):
    # selects tls security as highest supported method
    return b'\x03\x00\x00)$\xe0\x00\x00\x00\x00\x00Cookie: mstshash=xyz\r\n\x01\x00\x08\x00\x01\x00\x00\x00'

  @classmethod
  def ClientDataPDU(cls):
    tpkt = b'\x03\x00\x01\x8b'
    cc_header = b'\x02\xf0\x80'
    data = b'\x7fe\x82\x01\x7f\x04\x01\x01\x04\x01\x01\x01\x01\xff0\x1a\x02\x01"\x02\x01\x02\x02\x01\x00\x02\x01\x01\x02\x01\x00\x02\x01\x01\x02\x03\x00\xff\xff\x02\x01\x020\x19\x02\x01\x01\x02\x01\x01\x02\x01\x01\x02\x01\x01\x02\x01\x00\x02\x01\x01\x02\x02\x04 \x02\x01\x020 \x02\x03\x00\xff\xff\x02\x03\x00\xfc\x17\x02\x03\x00\xff\xff\x02\x01\x01\x02\x01\x00\x02\x01\x01\x02\x03\x00\xff\xff\x02\x01\x02\x04\x82\x01\x19\x00\x05\x00\x14|\x00\x01\x81\x10\x00\x08\x00\x10\x00\x01\xc0\x00Duca\x81\x02\x01\xc0\xea\x00\x04\x00\x08\x00\x00\x04\x00\x03\x01\xca\x03\xaa\t\x04\x00\x00(\n\x00\x00p\x00o\x00p\x00-\x00o\x00s\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x04\x00\x00\x00\x00\x00\x00\x00\x0c\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x01\xca\x01\x00\x00\x00\x00\x00\x10\x00\x07\x00!\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x06\x00\x01\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x04\xc0\x0c\x00\r\x00\x00\x00\x00\x00\x00\x00\x02\xc0\x0c\x00\x00\x00\x00\x00\x00\x00\x00\x00'
    return tpkt + cc_header + data

  @classmethod
  def ErectDomainRequest(cls):
    tpkt = b'\x03\x00\x00\x0c'
    cc_header = b'\x02\xf0\x80'
    data = b'\x04\x01\x00\x01\x00'
    return tpkt + cc_header + data

  @classmethod
  def AttactUserRequest(cls):
    tpkt = b'\x03\x00\x00\x08'
    cc_header = b'\x02\xf0\x80'
    data = b'\x28'
    return tpkt + cc_header + data

  @classmethod
  def ChannelJoinRequest(cls, channel):
    tpkt = b'\x03\x00\x00\x0c'
    cc_header = b'\x02\xf0\x80'
    if channel == 1007:
      data = b'8\x00\x06\x03\xef'
    if channel == 1003:
      data = b'8\x00\x06\x03\xeb'
    return tpkt + cc_header + data

  @classmethod
  def ClientInfoPDU(cls):
    # This conatins credentials
    tpkt = b'\x03\x00\x01\x5f'
    cc_header = b'\x02\xf0\x80'
    data = b'd\x00\x06\x03\xebp\x81P@\x00\x00\x00\x00\x00\x00\x00\xfb\x07\t\x00\x02\x00\x08\x00\x12\x00\x00\x00\x00\x00\x00\x00\x00\x00x\x00x\x00x\x00\x00\x00\x00\x00m\x00y\x00p\x00a\x00s\x00s\x001\x002\x00\x00\x00\x00\x00\x00\x00\x00\x00\x02\x00\x16\x001\x002\x007\x00.\x000\x00.\x000\x00.\x001\x00\x00\x00\x00\x00B\x00C\x00:\x00\\\x00W\x00i\x00n\x00d\x00o\x00w\x00s\x00\\\x00S\x00y\x00s\x00t\x00e\x00m\x003\x002\x00\\\x00m\x00s\x00t\x00s\x00c\x00a\x00x\x00.\x00d\x00l\x00l\x00\x00\x00\x00\x00\xb6\xfe\xff\xffC\x00\x00\x00l\x00\x00\x00i\x00\x00\x00e\x00\x00\x00n\x00\x00\x00t\x00\x00\x00 \x00\x00\x00L\x00\x00\x00o\x00\x00\x00c\x00\x00\x00a\x00\x00\x00l\x00\x00\x00 \x00\x00\x00T\x00\x00\x00i\x00\x00\x00m\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00C\x00\x00\x00l\x00\x00\x00i\x00\x00\x00e\x00\x00\x00n\x00\x00\x00t\x00\x00\x00 \x00\x00\x00L\x00\x00\x00o\x00\x00\x00c\x00\x00\x00a\x00\x00\x00l\x00\x00\x00 \x00\x00\x00T\x00\x00\x00i\x00\x00\x00m\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x06\x00\x00\x00\x00\x00'
    return tpkt + cc_header + data

@unittest.skip("RDP is broken - see issue #144")
class RDPTests(unittest.TestCase):

  def create_cert_if_not_exists(self, pem_file):
    if not os.path.isfile(pem_file):
      cert, key = generate_self_signed_cert("US", "None", "None", "None",
                                            "None", "*", 365, 0)
      with open(pem_file, 'wb') as _pem_file:
        _pem_file.write(cert)
        _pem_file.write(key)

  def rdp_connect(self, ip_addr, port):
    s = socket.socket()
    s.connect((ip_addr, port))
    s.sendall(RDPClient.ConnectionRequestPDU())
    s.recv(1024)

    tls = ssl.wrap_socket(s, ssl_version=ssl.PROTOCOL_TLSv1_1)
    tls.sendall(RDPClient.ClientDataPDU())
    tls.recv(4096)

    tls.send(RDPClient.ErectDomainRequest())
    tls.sendall(RDPClient.AttactUserRequest())
    tls.recv(512)

    tls.sendall(RDPClient.ChannelJoinRequest(1007))
    tls.recv(1024)
    tls.sendall(RDPClient.ChannelJoinRequest(1003))
    tls.recv(1024)

    tls.sendall(RDPClient.ClientInfoPDU())
    res = b''
    res = tls.recv(512)

    if res != b'':
      return False
    return True

  def setUp(self):
    self.loop = asyncio.new_event_loop()
    asyncio.set_event_loop(None)

    self.reporting_relay = ReportingRelay()
    self.reporting_relay_task = self.loop.run_in_executor(
        None, self.reporting_relay.start)

  def tearDown(self):
    self.reporting_relay.stop()
    # We give reporting_relay a chance to be finished
    self.loop.run_until_complete(self.reporting_relay_task)

    self.server.close()
    self.loop.run_until_complete(self.server.wait_closed())

    self.loop.run_until_complete(cancel_all_pending_tasks(self.loop))
    self.loop.close()

  def test_invalid_login(self):
    """Tests if rdp server responds correctly to a invalid login attempt."""

    def rdp_login():
      try:
        res = self.rdp_connect('0.0.0.0', 8389)
      except Exception as e:
        print("+++EXCEPTION+++ \n", e)
        return e
      return res

    options = {
        "enabled": "True",
        "port": 3389,
        "protocol_specific_data": {
            "banner": "",
            "cert": {
                "common_name": "*",
                "country": "US",
                "state": "None",
                "locality": "None",
                "organization": "None",
                "organizational_unit": "None",
                "valid_days": 365,
                "serial_number": 0
            }
        }
    }
    rdp_cap = rdp.RDP(options, self.loop)
    self.create_cert_if_not_exists('rdp.pem')
    server_coro = asyncio.start_server(
        rdp_cap.handle_session, '0.0.0.0', 8389, loop=self.loop)
    self.server = self.loop.run_until_complete(server_coro)

    rdp_task = self.loop.run_in_executor(None, rdp_login)
    login_res = self.loop.run_until_complete(rdp_task)

    self.assertEqual(True, login_res)



# Copyright (C) 2017 Johnny Vestergaard <jkv@unixcluster.dk>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import asyncio
import unittest

from heralding.capabilities.ssh import SSH
from heralding.reporting.reporting_relay import ReportingRelay

import asyncssh


class SshTests(unittest.TestCase):

  def setUp(self):
    self.loop = asyncio.new_event_loop()
    asyncio.set_event_loop(None)

    self.reporting_relay = ReportingRelay()
    self.reporting_relay_task = self.loop.run_in_executor(
        None, self.reporting_relay.start)

  def tearDown(self):
    self.reporting_relay.stop()
    # We give reporting_relay a chance to be finished
    self.loop.run_until_complete(self.reporting_relay_task)

    self.server.close()
    self.loop.run_until_complete(self.server.wait_closed())

    self.loop.close()

  def test_basic_login(self):

    async def run_client():
      async with asyncssh.connect(
          'localhost',
          port=8888,
          username='johnny',
          password='secretpw',
          known_hosts=None) as _:
        pass

    ssh_key_file = 'ssh.key'
    SSH.generate_ssh_key(ssh_key_file)

    options = {'enabled': 'True', 'port': 8888}
    server_coro = asyncssh.create_server(
        lambda: SSH(options, self.loop),
        '0.0.0.0',
        8888,
        server_host_keys=['ssh.key'])
    self.server = self.loop.run_until_complete(server_coro)

    try:
      self.loop.run_until_complete(run_client())
    except asyncssh.misc.PermissionDenied:
      pass



# Copyright (C) 2013 Aniket Panse <contact@aniketpanse.in>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# Parts of this code are from secure-smtpd (https://github.com/bcoe/secure-smtpd)

# Aniket Panse <contact@aniketpanse.in> grants Johnny Vestergaard <jkv@unixcluster.dk>
# a perpetual, worldwide, non-exclusive, no-charge, royalty-free, irrevocable
# copyright license to reproduce, prepare derivative works of, publicly
# display, publicly perform, sublicense, relicense, and distribute [the] Contributions
# and such derivative works.

import time
import random
import base64
import socket
import asyncio
import logging

from aiosmtpd.smtp import SMTP, MISSING, syntax

from heralding.capabilities.handlerbase import HandlerBase

log = logging.getLogger(__name__)


class SMTPHandler(SMTP):
  fqdn = ''

  def __init__(self, reader, writer, session, options, loop):
    self.banner = options['protocol_specific_data']['banner']
    super().__init__(None, hostname=self.banner, loop=loop)
    # Reset standard banner.
    self.__ident__ = ""
    self._reader = reader
    self._writer = writer
    self.transport = writer

    self._set_rset_state()
    self.session = session
    self.session.peer = self.transport.get_extra_info('peername')
    self.session.extended_smtp = None
    self.session.host_name = None

  async def push(self, status):
    response = bytes(status + '\r\n',
                     'utf-8' if self.enable_SMTPUTF8 else 'ascii')
    self._writer.write(response)
    log.debug(response)
    try:
      await self._writer.drain()
    except ConnectionResetError:
      self.stop()
    if self._reader.at_eof():
      self.stop()

  @syntax('EHLO hostname')
  async def smtp_EHLO(self, hostname):
    if not hostname:
      await self.push('501 Syntax: EHLO hostname')
      return
    self._set_rset_state()
    await self.push('250-{0} Hello {1}'.format(self.hostname, hostname))
    await self.push('250-AUTH PLAIN LOGIN CRAM-MD5')
    await self.push('250 EHLO')

  @syntax("AUTH mechanism [initial-response]")
  async def smtp_AUTH(self, arg):
    if not arg:
      await self.push('500 Not enough values')
      return
    args = arg.split()
    if len(args) > 2:
      await self.push('500 Too many values')
      return
    mechanism = args[0]
    if mechanism == 'PLAIN':
      if len(args) == 1:
        await self.push('334 ')  # wait for client login/password
        line = await self.readline()
        if not line:
          return
        blob = line.strip()
      else:
        blob = args[1].encode()

      try:
        loginpassword = base64.b64decode(blob)
      except Exception:
        await self.push("501 Can't decode base64")
        return
      try:
        _, login, password = loginpassword.split(b"\x00")
      except ValueError:  # not enough args
        await self.push("500 Can't split auth value")
        return
      self.session.add_auth_attempt(
          'PLAIN',
          username=str(login, 'utf-8'),
          password=str(password, 'utf-8'))
    elif mechanism == 'LOGIN':
      if len(args) > 1:
        username = str(base64.b64decode(args[1]), 'utf-8')
        await self.push('334 ' + str(base64.b64encode(b'Password:'), 'utf-8'))

        password_bytes = await self.readline()
        if not password_bytes:
          return
        password = str(base64.b64decode(password_bytes), 'utf-8')
        self.session.add_auth_attempt(
            'LOGIN', username=username, password=password)
      else:
        await self.push('334 ' + str(base64.b64encode(b'Username:'), 'utf-8'))

        username_bytes = await self.readline()
        if not username_bytes:
          return

        await self.push('334 ' + str(base64.b64encode(b'Password:'), 'utf-8'))

        password_bytes = await self.readline()
        if not password_bytes:
          return
        self.session.add_auth_attempt(
            'LOGIN',
            username=str(base64.b64decode(username_bytes), 'utf-8'),
            password=str(base64.b64decode(password_bytes), 'utf-8'))
    elif mechanism == 'CRAM-MD5':
      r = random.randint(5000, 20000)
      t = int(time.time())

      # challenge is of the form '<24609.1047914046@awesome.host.com>'
      sent_cram_challenge = "<" + str(r) + "." + str(
          t) + "@" + SMTPHandler.fqdn + ">"
      cram_challenge_bytes = bytes(sent_cram_challenge, 'utf-8')
      await self.push("334 " +
                      str(base64.b64encode(cram_challenge_bytes), 'utf-8'))

      credentials_bytes = await self.readline()
      if not credentials_bytes:
        return
      credentials = str(base64.b64decode(credentials_bytes), 'utf-8')
      if sent_cram_challenge is None or ' ' not in credentials:
        await self.push('451 Internal confusion')
        return
      username, digest = credentials.split()
      self.session.add_auth_attempt(
          'cram_md5',
          username=username,
          digest=digest,
          challenge=sent_cram_challenge)
      await self.push('535 authentication failed')
    else:
      await self.push('500 incorrect AUTH mechanism')
      return
    status = '535 authentication failed'
    await self.push(status)

  @syntax('QUIT')
  async def smtp_QUIT(self, arg):
    if arg:
      await self.push('501 Syntax: QUIT')
    else:
      status = await self._call_handler_hook('QUIT')
      await self.push('221 Bye' if status is MISSING else status)
      self.stop()

  async def readline(self):
    line = b''
    try:
      line = await self._reader.readline()
    except ConnectionResetError:
      self.stop()
    else:
      return line

  def stop(self):
    if self.transport is not None:
       self.transport.close()
    self.transport = None

  def _timeout_cb(self):
    if self.transport is not None:
       super()._timeout_cb()

class smtp(HandlerBase):

  def __init__(self, options, loop):
    super().__init__(options, loop)
    self.loop = loop
    self._options = options

  async def execute_capability(self, reader, writer, session):
    fqdn_task = asyncio.ensure_future(self.setfqdn(), loop=self.loop)

    smtp_cap = SMTPHandler(reader, writer, session, self._options, self.loop)
    smtp_task = asyncio.ensure_future(smtp_cap._handle_client(), loop=self.loop)

    await smtp_task

    fqdn_task.cancel()
    try:
      await fqdn_task
    except asyncio.CancelledError:
      pass

  async def setfqdn(self):
    if 'fqdn' in self._options['protocol_specific_data'] and self._options[
        'protocol_specific_data']['fqdn']:
      SMTPHandler.fqdn = self._options['protocol_specific_data']['fqdn']
    else:
      while True:
        fqdn = await self.loop.run_in_executor(None, socket.getfqdn)
        SMTPHandler.fqdn = fqdn
        await asyncio.sleep(1800, loop=self.loop)



# Copyright (C) 2013 Aniket Panse <contact@aniketpanse.in>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import logging
import binascii

from heralding.capabilities.handlerbase import HandlerBase
from heralding.libs.cracker.vnc import crack_hash

# VNC constants
RFB_VERSION = b'RFB 003.007\n'
AUTH_METHODS = b'\x01\x02'
VNC_AUTH = b'\x02'
AUTH_FAILED = b'\x00\x00\x00\x01'

logger = logging.getLogger(__name__)


class Vnc(HandlerBase):

  async def execute_capability(self, reader, writer, session):
    await self._handle_session(reader, writer, session)

  async def _handle_session(self, reader, writer, session):
    writer.write(RFB_VERSION)
    client_version = await reader.read(1024)

    if client_version == RFB_VERSION:
      await self.security_handshake(reader, writer, session)
    else:
      session.end_session()

  async def security_handshake(self, reader, writer, session):
    writer.write(AUTH_METHODS)
    sec_method = await reader.read(1024)

    if sec_method == VNC_AUTH:
      await self.do_vnc_authentication(reader, writer, session)
    else:
      session.end_session()

  async def do_vnc_authentication(self, reader, writer, session):
    challenge = os.urandom(16)
    writer.write(challenge)

    client_response = await reader.read(1024)
    writer.write(AUTH_FAILED)

    # try to decrypt the hash
    dkey = crack_hash(challenge, client_response)
    if dkey:
      session.add_auth_attempt('cracked', password=dkey)
    else:
      hash_data = {
          'challenge': binascii.hexlify(challenge).decode(),
          'response': binascii.hexlify(client_response).decode()
      }
      session.add_auth_attempt('des_challenge', password_hash=hash_data)

    session.end_session()



import binascii
from Crypto.Cipher import DES

import heralding


def get_vnc_key(key):
  bit_flip = [
      0x00, 0x80, 0x40, 0xC0, 0x20, 0xA0, 0x60, 0xE0, 0x10, 0x90, 0x50, 0xD0,
      0x30, 0xB0, 0x70, 0xF0, 0x08, 0x88, 0x48, 0xC8, 0x28, 0xA8, 0x68, 0xE8,
      0x18, 0x98, 0x58, 0xD8, 0x38, 0xB8, 0x78, 0xF8, 0x04, 0x84, 0x44, 0xC4,
      0x24, 0xA4, 0x64, 0xE4, 0x14, 0x94, 0x54, 0xD4, 0x34, 0xB4, 0x74, 0xF4,
      0x0C, 0x8C, 0x4C, 0xCC, 0x2C, 0xAC, 0x6C, 0xEC, 0x1C, 0x9C, 0x5C, 0xDC,
      0x3C, 0xBC, 0x7C, 0xFC, 0x02, 0x82, 0x42, 0xC2, 0x22, 0xA2, 0x62, 0xE2,
      0x12, 0x92, 0x52, 0xD2, 0x32, 0xB2, 0x72, 0xF2, 0x0A, 0x8A, 0x4A, 0xCA,
      0x2A, 0xAA, 0x6A, 0xEA, 0x1A, 0x9A, 0x5A, 0xDA, 0x3A, 0xBA, 0x7A, 0xFA,
      0x06, 0x86, 0x46, 0xC6, 0x26, 0xA6, 0x66, 0xE6, 0x16, 0x96, 0x56, 0xD6,
      0x36, 0xB6, 0x76, 0xF6, 0x0E, 0x8E, 0x4E, 0xCE, 0x2E, 0xAE, 0x6E, 0xEE,
      0x1E, 0x9E, 0x5E, 0xDE, 0x3E, 0xBE, 0x7E, 0xFE, 0x01, 0x81, 0x41, 0xC1,
      0x21, 0xA1, 0x61, 0xE1, 0x11, 0x91, 0x51, 0xD1, 0x31, 0xB1, 0x71, 0xF1,
      0x09, 0x89, 0x49, 0xC9, 0x29, 0xA9, 0x69, 0xE9, 0x19, 0x99, 0x59, 0xD9,
      0x39, 0xB9, 0x79, 0xF9, 0x05, 0x85, 0x45, 0xC5, 0x25, 0xA5, 0x65, 0xE5,
      0x15, 0x95, 0x55, 0xD5, 0x35, 0xB5, 0x75, 0xF5, 0x0D, 0x8D, 0x4D, 0xCD,
      0x2D, 0xAD, 0x6D, 0xED, 0x1D, 0x9D, 0x5D, 0xDD, 0x3D, 0xBD, 0x7D, 0xFD,
      0x03, 0x83, 0x43, 0xC3, 0x23, 0xA3, 0x63, 0xE3, 0x13, 0x93, 0x53, 0xD3,
      0x33, 0xB3, 0x73, 0xF3, 0x0B, 0x8B, 0x4B, 0xCB, 0x2B, 0xAB, 0x6B, 0xEB,
      0x1B, 0x9B, 0x5B, 0xDB, 0x3B, 0xBB, 0x7B, 0xFB, 0x07, 0x87, 0x47, 0xC7,
      0x27, 0xA7, 0x67, 0xE7, 0x17, 0x97, 0x57, 0xD7, 0x37, 0xB7, 0x77, 0xF7,
      0x0F, 0x8F, 0x4F, 0xCF, 0x2F, 0xAF, 0x6F, 0xEF, 0x1F, 0x9F, 0x5F, 0xDF,
      0x3F, 0xBF, 0x7F, 0xFF
  ]

  # pad or trucate upto 8 bytes
  key = (key + bytes(8))[:8]
  res = bytearray(8)
  i = 0
  for b in key:
    res[i] = bit_flip[b]
    i += 1

  return bytes(res)


def vnc_hash_check(challenge, response, vnc_key):
  """ @param: all params in byte encoded
            @return: True if decrypted response matches with challenge"""
  cipher = DES.new(vnc_key, DES.MODE_ECB)
  drs = cipher.decrypt(response)
  if drs == challenge:
    return True
  else:
    return False


def crack_hash(challenge, response):
  # to overcome circular import
  password_list = heralding.honeypot.Honeypot.wordlist

  if password_list:
    for potential_password in password_list:
      vnc_key = get_vnc_key(potential_password.encode('ascii'))
      if (vnc_hash_check(challenge, response, vnc_key)):
        return potential_password
  return None
